/*
TrussTopologyRobustRedundant

Author: Daniel Mohr
Date: 2012-02-10, 2012-02-11, 2012-02-12, 2012-02-21, 2012-02-26, 2012-02-29

License: GNU GENERAL PUBLIC LICENSE, Version 3, 29 June 2007

Copyright (C) 2012 Daniel Mohr
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License as
published by the Free Software Foundation; either version 3 of
the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, see
http://www.gnu.org/licenses/
*/

#include <stdio.h>
#include <stdlib.h>
#include <float.h>
#include <string.h>
#include <assert.h>
#include <math.h>
#include <glpk.h>

#include "ttrr_structs.h"
#include "ttrr_lesen.h"
#include "ttrr_schreiben.h"
#include "ttrr_tools.h"
#include "ttrr_geometriematrix_stablaengen.h"
#include "ttrr_berechne_freedofs.h"
#include "ttrr_beschraenke_geometriematrix_auf_freedofs.h"
#include "ttrr_beschraenke_kraefte_auf_freedofs.h"
#include "ttrr_berechne_zielfunktion.h"
#include "ttrr_berechne_gleichungen.h"
#include "ttrr_berechne_ungleichungen.h"
#include "ttrr_berechne_lb_ub.h"
#include "ttrr_erstelle_lp.h"
/* #include "ttrr_skaliere_steilster_abstieg.h" */
#include "ttrr_cb_mip_greedy.h"
#include "ttrr_scale.h"

int main(int argc, char *argv[])
{
  int dim,redundanz,opt;
  double smax;
  struct double_matrix knoten; /* struct double_matrix{int n,m;double **val;}; */
  struct int_matrix staebe; /* struct int_matrix{int n,m;int **val;}; */
  struct int_vektor lager,freedofs; /* struct int_vektor{int i,n;int *val;}; */
  struct double_vektor z,beq,b,lb,ub; /* struct double_vektor{int i,n;double *val;}; */
  struct sparse_matrix kraefte,C,Aeq,A; /* struct sparse_matrix{int i,n,m,nz,*ia,*ja;double *ar;}; */
  char *name = NULL;
  double dichte,sigmamin,sigmamax,zlb,zub;
  struct double_vektor laengen;
  glp_prob *lp = NULL, *lpmip = NULL;
  int ret;
  /* double alpha,beta,gamma; */
  struct double_vektor solution,w,s;
  struct int_vektor x;
  struct cb_info cb_info;
  int sort;
  /* double *fs = NULL, *fz = NULL; */
  int i;
  glp_smcp parameter_smcp;
  glp_iptcp parameter_iptcp;
  glp_iocp parameter_iocp;
  double zielfktwert;
  double scale_sigma,scale_kraefte;
  initialisiere_null_double_matrix(&knoten);
  initialisiere_null_int_matrix(&staebe);
  initialisiere_null_int_vektor(&lager); initialisiere_null_int_vektor(&freedofs);
  initialisiere_null_double_vektor(&z);
  initialisiere_null_double_vektor(&beq); initialisiere_null_double_vektor(&b);
  initialisiere_null_double_vektor(&lb); initialisiere_null_double_vektor(&ub);
  initialisiere_null_sparse_matrix(&kraefte);
  initialisiere_null_sparse_matrix(&C);
  initialisiere_null_sparse_matrix(&A); initialisiere_null_sparse_matrix(&Aeq);
  initialisiere_null_double_vektor(&solution);
  initialisiere_null_double_vektor(&w); initialisiere_null_double_vektor(&s);
  initialisiere_null_int_vektor(&x);
  printf("### ttrr ### gestartet ###\n");
  if (argc < 2)
    {
      printf("zu wenig Parameter\n");
      printf("argc = %d\n",argc);
      exit(EXIT_FAILURE);
    }
  lp = glp_create_prob();
  lpmip = glp_create_prob();
  /* lese Problem */
  lese_int_from(argv[1],"dim",&dim);
  lese_matrix_double(argv[1],"knoten",&knoten);
  lese_matrix_int(argv[1],"staebe",&staebe);
  lese_vektor(argv[1],"lager",&lager);
  lese_sparse_matrix(argv[1],"kraefte",&kraefte);
  lese_double_from(argv[1],"smax",&smax);
  lese_string(argv[1],"info.txt",&name,100);
  lese_int_from(argv[1],"redundanz",&redundanz);
  lese_int_from(argv[1],"opt",&opt);
  lese_int_from(argv[1],"sort",&sort);
  lese_double_from(argv[1],"sigmamin",&sigmamin);
  lese_double_from(argv[1],"sigmamax",&sigmamax);
  lese_double_from(argv[1],"dichte",&dichte);
  lese_double_from(argv[1],"zlb",&zlb);
  lese_double_from(argv[1],"zub",&zub);
  lese_double_from(argv[1],"scale_sigma",&scale_sigma);
  lese_double_from(argv[1],"scale_kraefte",&scale_kraefte);
  /* Infos ausgeben */
  printf("%s: dim = %d ; smax = %e ; redundanz = %d ; opt = %d ; sort = %d\n",name,dim,smax,redundanz,opt,sort);
  printf("sigmamin = %e ; sigmamax = %e ; dichte = %e\n",sigmamin,sigmamax,dichte);
  printf("%d Knoten ; %d Staebe ; %d Lager ; %d Lastfaelle\n",knoten.n,staebe.n,lager.n,kraefte.m);
  /* Vorbereitung der Rechnung */
  scale(&scale_sigma,&scale_kraefte,&kraefte,&sigmamin,&sigmamax,&smax);
  geometriematrix_stablaengen(&C,&laengen,&knoten,&staebe);
  schreibe_double_vektor(argv[1],"laengen",&laengen);
  schreibe_sparse_matrix(argv[1],"C",&C);
  /* print_int_vektor("lager",&lager); */
  berechne_freedofs(&freedofs,0,dim*knoten.n,&lager);
  schreibe_int_vektor(argv[1],"freedofs",&freedofs);
  /* print_int_vektor("freedofs",&freedofs); */
  beschraenke_geometriematrix_auf_freedofs(&C,&freedofs);
  schreibe_sparse_matrix(argv[1],"C_freedofs",&C);
  beschraenke_kraefte_auf_freedofs(&kraefte,&freedofs);
  schreibe_sparse_matrix(argv[1],"kraefte_freedofs",&kraefte);
  /* erstelle Problem */
  printf("erstelle Optimierungsproblem\n");
  berechne_zielfunktion(&z,redundanz,&kraefte,&staebe,&laengen);
  /* print_double_vektor("z",&z); */
  berechne_gleichungen(&Aeq,&beq,redundanz,&kraefte,&staebe,&laengen,&C);
  schreibe_sparse_matrix(argv[1],"Aeq",&Aeq);
  schreibe_double_vektor(argv[1],"beq",&beq);
  berechne_ungleichungen(&A,&b,redundanz,&kraefte,&staebe,&laengen,sigmamin,sigmamax,smax,sort,&z,zlb,zub);
  schreibe_sparse_matrix(argv[1],"A",&A);
  schreibe_double_vektor(argv[1],"b",&b);
  berechne_lb_ub(&lb,&ub,redundanz,&kraefte,&staebe,&laengen,sigmamin,sigmamax,smax);
  schreibe_double_vektor(argv[1],"lb",&lb);
  schreibe_double_vektor(argv[1],"ub",&ub);
  if (redundanz==0)
    {
      erstelle_lp(lp,name,&A,&b,&Aeq,&beq,&lb,&ub,&z);
      schreibe_lp(argv[1],lp,name);
    } else {
    erstelle_lpmip(lpmip,name,&A,&b,&Aeq,&beq,&lb,&ub,&z,redundanz,&staebe);
    schreibe_lp(argv[1],lpmip,name);
  }
  fflush(NULL);
  /* Rechnung */
  printf("berechne...\n");fflush(NULL);
  x.n = abs(redundanz) * staebe.n;
  if (redundanz != 0)
    {
      x.val = (int*)calloc(x.n,sizeof(int));assert(x.val != NULL);
      w.n = kraefte.m * abs(redundanz) * staebe.n; /* Anzahl innerer Stabkraefte */
      s.n = abs(redundanz) * staebe.n; /* Anzahl Stabdicken */
    } else {
    w.n = kraefte.m * staebe.n; /* Anzahl innerer Stabkraefte */
    s.n = staebe.n; /* Anzahl Stabdicken */
  }
  w.val = (double*)calloc(w.n,sizeof(double));assert(w.val != NULL);
  s.val = (double*)calloc(s.n,sizeof(double));assert(s.val != NULL);
  solution.n = x.n+w.n+s.n;
  solution.val = (double*)calloc(solution.n,sizeof(double));assert(solution.val != NULL);
  if (redundanz==0)
    {
      if (opt == 0)
	{
	  /* opt == 0: glp_simplex */
	  printf("opt == 0: glp_simplex\n");fflush(NULL);
	  glp_init_smcp(&parameter_smcp);
	  parameter_smcp.presolve = GLP_ON; /* enable using the LP presolver */
	  parameter_smcp.msg_lev = GLP_MSG_ALL; /* full output (including informational messages) */
	  ret = glp_simplex(lp,&parameter_smcp);fflush(NULL);
	  for(i = 0; i < solution.n; i++)
	    {
	      solution.val[i] = glp_get_col_prim(lp,i+1);
	      if (i < w.n)
		w.val[i] = solution.val[i];
	      else if (w.n <= i)
		s.val[i-w.n] = solution.val[i];
	    }
	} else {
	/* opt == 1: glp_interior */
	printf("opt == 1: glp_interior\n");fflush(NULL);
	glp_init_iptcp(&parameter_iptcp);
	parameter_iptcp.msg_lev = GLP_MSG_ALL; /* full output (including informational messages) */
	glp_scale_prob(lp,GLP_SF_AUTO);
	ret = glp_interior(lp,&parameter_iptcp);fflush(NULL);
	for(i = 0; i < solution.n; i++)
	  {
	    solution.val[i] = glp_ipt_col_prim(lp,i+1);
	    if (i < w.n)
	      w.val[i] = solution.val[i];
	    else if (w.n <= i)
	      s.val[i-w.n] = solution.val[i];
	  }
      }
    schreibe_lp_solution(argv[1],lp,name,redundanz,opt);
    } else {
    /* glp_intopt */
    printf("glp_intopt\n");fflush(NULL);
    glp_init_iocp(&parameter_iocp);
    parameter_iocp.br_tech = GLP_BR_PCH; /* hybrid pseudocost heuristic */
    parameter_iocp.bt_tech = GLP_BT_BPH; /* best projection heuristic */
    parameter_iocp.presolve = GLP_ON; /* enable using the LP presolver */
    parameter_iocp.cov_cuts = GLP_ON;
    parameter_iocp.binarize = GLP_ON;
    if (opt >= 3)
      {
	printf("setze callback-Routine greedy\n");
	cb_info.opt = opt;
	cb_info.n = redundanz;
	cb_info.N = staebe.n;
	cb_info.nl = kraefte.m;
	cb_info.nx = x.n;
	cb_info.nw = w.n;
	cb_info.ns = s.n;
	cb_info.mipsols = 0;
	cb_info.solution.n = solution.n;
	cb_info.solution.val = (double*)calloc(cb_info.solution.n,sizeof(double));assert(cb_info.solution.val != NULL);
	cb_info.x.n = x.n;
	cb_info.x.val = (int*)calloc(cb_info.x.n,sizeof(int));assert(cb_info.x.val != NULL);
	cb_info.w.n = w.n;
	cb_info.w.val = (double*)calloc(cb_info.w.n,sizeof(double));assert(cb_info.w.val != NULL);
	cb_info.s.n = s.n;
	cb_info.s.val = (double*)calloc(cb_info.s.n,sizeof(double));assert(cb_info.s.val != NULL);
	cb_info.pfad = argv[1];
	parameter_iocp.cb_func = cb_mip_greedy;
	parameter_iocp.cb_info = &cb_info;
      }
    ret = glp_intopt(lpmip,&parameter_iocp);fflush(NULL);
    printf("%d mip-solutions found\n",cb_info.mipsols);
    hole_mip_solution(&solution,&x,&w,&s,lpmip);
    schreibe_lp_solution(argv[1],lpmip,name,redundanz,opt);
  }
  unscale(&scale_sigma,&scale_kraefte,&kraefte,&sigmamin,&sigmamax,&smax,&w,&s,&solution);
  /* schreibe Problemloesung */
  zielfktwert = 0;
  for(i = 0; i < solution.n; i++)
    zielfktwert += z.val[i] * solution.val[i];
  printf("Zielfunktionswert: %f\n",zielfktwert);
  printf("Gesamtvolumen: %f\n",zielfktwert);
  printf("Gesamtgewicht: %f\n",zielfktwert*dichte);
  if (redundanz != 0)
    schreibe_int_vektor(argv[1],"x",&x);
  schreibe_double_vektor(argv[1],"w",&w);
  schreibe_double_vektor(argv[1],"s",&s);
  schreibe_double_file(argv[1],"zielfktwert",&zielfktwert);
  printf("gebe Speicher wieder frei\n");fflush(NULL);
  free(knoten.val);
  free(staebe.val);
  free(lager.val);
  free(kraefte.ia);
  free(kraefte.ja);
  free(kraefte.ar);
  free(laengen.val);
  free(freedofs.val);
  free(z.val);
  free(Aeq.ia);
  free(Aeq.ja);
  free(Aeq.ar);
  free(beq.val);
  free(A.ia);
  free(A.ja);
  free(A.ar);
  free(b.val);
  free(lb.val);
  free(ub.val);
  glp_delete_prob(lp);
  glp_delete_prob(lpmip);
  free(x.val);
  free(w.val);
  free(s.val);
  free(solution.val);
  printf("### ttrr ### beendet ###\n");fflush(NULL);
  exit(EXIT_SUCCESS);
}
