/*
TrussTopologyRobustRedundant - berechne_ungleichungen

Author: Daniel Mohr
Date: 2012-02-29
*/

#include "ttrr_berechne_ungleichungen.h"

void berechne_ungleichungen(struct sparse_matrix *A,struct double_vektor *b,const int redundanz,const struct sparse_matrix *kraefte,const struct int_matrix *staebe,const struct double_vektor *laengen,const double sigmamin,const double sigmamax,const double smax,const int sort,struct double_vektor *z, double zlb, double zub)
{
  int red = abs(redundanz);
  unsigned char isred = 1;
  int nx; /* Anzahl binaerer Variablen */
  int nw; /* Anzahl innerer Stabkraefte */
  int ns; /* Anzahl Stabdicken */
  int l,r,i,j;
  if (red == 0)
    {
      red = 1;
      isred = 0;
    }
  nx = abs(redundanz) * staebe->n; /* Anzahl binaerer Variablen */
  nw = kraefte->m * red * staebe->n; /* Anzahl innerer Stabkraefte */
  ns = red * staebe->n; /* Anzahl Stabdicken */
  A->n = 2 * nw + isred * staebe->n + nx + isred * sort * (red-1); /* Anzahl Zeilen */
  A->m = nx + nw + ns; /* Anzahl Spalten */
  if (zlb > 0) A->n++;
  if (zub > 0) A->n++;
  b->n = A->n;
  b->val = (double*)calloc(b->n,sizeof(double));assert(b->val != NULL);
  A->i = 0;
  b->i = 0;
  /* struct sparse_matrix{int i,n,m,nz,*ia,*ja;double *ar;}; */
  if (redundanz <= 0)
    {
      /* redundanz == 0 oder "1-1/n"-Redundanz */
      if (redundanz != 0)
	printf("berechne Unleichungen (\"1-1/n\"-Redundanz)...");
      else
	printf("berechne Unleichungen...");fflush(NULL);
      fflush(NULL);
      A->nz = 2 * (2 * nw) + abs(redundanz) * staebe->n + isred * 2 * nx + isred * sort * (red-1) * staebe->n * 2; /* Anzahl Nicht-Null-Elemente */
      if (zlb > 0) A->nz += A->m;
      if (zub > 0) A->nz += A->m;
      definiere_sparse_matrix(A);
      /* sigmamin s_i <= w_i^(k) <= sigmamax s_i */
      /* Anzahl Ungleichungen: 2 * nw = 2 * kraefte->m * red * staebe->n */
      for(l = 0; l < kraefte->m; l++)
	for(r = 0; r < red; r++)
	  {
	    for(i = 0; i < staebe->n; i++)
	      {
		/* sigmamin s_i <= w_i^(k) */
		A->ia[A->i] = b->i; /* Zeile */
		A->ja[A->i] = nx + l * red * staebe->n + r * staebe->n + i; /* Spalte */
		A->ar[A->i] = -1.0; /* Wert */
		A->i++;
		A->ia[A->i] = b->i; /* Zeile */
		A->ja[A->i] = nx + nw + r * staebe->n + i; /* Spalte */
		A->ar[A->i] = sigmamin; /* Wert */
		A->i++;
		b->val[b->i] = 0;
		b->i++;
		/* w_i^(k) <= sigmamax s_i */
		A->ia[A->i] = b->i; /* Zeile */
		A->ja[A->i] = nx + l * red * staebe->n + r * staebe->n + i; /* Spalte */
		A->ar[A->i] = 1.0; /* Wert */
		A->i++;
		A->ia[A->i] = b->i; /* Zeile */
		A->ja[A->i] = nx + nw + r * staebe->n + i; /* Spalte */
		A->ar[A->i] = -sigmamax; /* Wert */
		A->i++;
		b->val[b->i] = 0;
		b->i++;
	      }
	  }
    } else {
    /* redundanz > 0: "1/n"-Redundanz */
    printf("berechne Unleichungen (\"1/n\"-Redundanz)...");fflush(NULL);
    A->nz = (1+red-1) * (2 * nw) + abs(redundanz) * staebe->n + isred * 2 * nx + isred * sort * (red-1) * staebe->n * 2; /* Anzahl Nicht-Null-Elemente */
    if (zlb > 0) A->nz += A->m;
    if (zub > 0) A->nz += A->m;
    definiere_sparse_matrix(A);
    /* sigmamin ns_i <= w_i^(k) <= sigmamax ns_i */
    /* Anzahl Ungleichungen: 2 * nw = 2 * kraefte->m * red * staebe->n */
    for(l = 0; l < kraefte->m; l++)
      for(r = 0; r < red; r++)
	{
	  for(i = 0; i < staebe->n; i++)
	    {
	      /* sigmamin s_i <= w_i^(k) */
	      A->ia[A->i] = b->i; /* Zeile */
	      A->ja[A->i] = nx + l * red * staebe->n + r * staebe->n + i; /* Spalte */
	      A->ar[A->i] = -1.0; /* Wert */
	      A->i++;
	      for(j = 0; j < red; j++)
		{
		  if (j != r)
		    {
		      A->ia[A->i] = b->i; /* Zeile */
		      A->ja[A->i] = nx + nw + j * staebe->n + i; /* Spalte */
		      A->ar[A->i] = sigmamin; /* Wert */
		      A->i++;
		    }
		}
	      b->val[b->i] = 0;
	      b->i++;
	      /* w_i^(k) <= sigmamax s_i */
	      A->ia[A->i] = b->i; /* Zeile */
	      A->ja[A->i] = nx + l * red * staebe->n + r * staebe->n + i; /* Spalte */
	      A->ar[A->i] = 1.0; /* Wert */
	      A->i++;
	      for(j = 0; j < red; j++)
		{
		  if (j != r)
		    {
		      A->ia[A->i] = b->i; /* Zeile */
		      A->ja[A->i] = nx + nw + j * staebe->n + i; /* Spalte */
		      A->ar[A->i] = -sigmamax; /* Wert */
		      A->i++;
		    }
		}
	      b->val[b->i] = 0;
	      b->i++;
	    }
	}
  }
  /* sum(x_i) <= 1 */
  /* Anzahl Ungleichungen: isred * staebe->n */
  if (redundanz != 0)
    for(i = 0; i < staebe->n; i++)
      {
	for(r = 0; r < red; r++)
	  {
	    A->ia[A->i] = b->i; /* Zeile */
	    A->ja[A->i] = r * staebe->n + i; /* Spalte */
	    A->ar[A->i] = 1.0; /* Wert */
	    A->i++;
	  }
	b->val[b->i] = 1;
	b->i++;
      }
  /* s_i <= smax x_i */
  /* Anzahl Ungleichungen: nx = abs(redundanz) * staebe->n */
  if (redundanz != 0)
    for(i = 0; i < nx; i++)
      {
	A->ia[A->i] = b->i; /* Zeile */
	A->ja[A->i] = i; /* Spalte */
	A->ar[A->i] = -smax; /* Wert */
	A->i++;
	A->ia[A->i] = b->i; /* Zeile */
	A->ja[A->i] = nx + nw + i; /* Spalte */
	A->ar[A->i] = 1.0; /* Wert */
	A->i++;
	b->val[b->i] = 0;
	b->i++;
      }
  if ((sort == 1) && (redundanz != 0))
    {
      printf("[sort=1]...");
      /* erstelle Ungleichungen, damit die Teilstrukturen im Volumen aufsteigend */
      /* Anzahl Ungleichungen: isred * sort * (red-1) */
      /* nz: isred * sort * (red-1) * staebe->n * 2 */
      for(r = 0; r < red-1; r++)
	{ /* l^T s_i <= l^T s_(i+1) */
	  for(i = 0; i < staebe->n; i++)
	    {
	      A->ia[A->i] = b->i; /* Zeile */
	      A->ja[A->i] = nx + nw + r * staebe->n + i; /* Spalte */
	      A->ar[A->i] = laengen->val[i]; /* Wert */
	      A->i++;
	      A->ia[A->i] = b->i; /* Zeile */
	      A->ja[A->i] = nx + nw + (r+1) * staebe->n + i; /* Spalte */
	      A->ar[A->i] = -laengen->val[i]; /* Wert */
	      A->i++;
	    }
	  b->val[b->i] = 0;
	  b->i++;
	}

    }
  if (zlb > 0)
    { /* erstelle zusaetzliche untere Schranke fuer die Zielfunktion */
      printf("[zlb=%e]...",zlb);
      for(i = 0; i < A->m; i++)
	{
	  if (z->val[i] != 0)
	    {
	      A->ia[A->i] = b->i; /* Zeile */
	      A->ja[A->i] = i; /* Spalte */
	      A->ar[A->i] = -z->val[i]; /* Wert */
	      A->i++;
	    }
	}
      b->val[b->i] = -zlb;
      b->i++;
    }
  if (zub > 0)
    { /* erstelle zusaetzliche obere Schranke fuer die Zielfunktion */
      printf("[zub=%e]...",zub);
      for(i = 0; i < A->m; i++)
	{
	  if (z->val[i] != 0)
	    {
	      A->ia[A->i] = b->i; /* Zeile */
	      A->ja[A->i] = i; /* Spalte */
	      A->ar[A->i] = z->val[i]; /* Wert */
	      A->i++;
	    }
	}
      b->val[b->i] = zub;
      b->i++;
    }
  if ( A->i < A->nz)
    {
      A->nz = A->i;
      A->ia = (int*)realloc(A->ia,A->nz * sizeof(int));assert(A->ia != NULL);
      A->ja = (int*)realloc(A->ja,A->nz * sizeof(int));assert(A->ja != NULL);
      A->ar = (double*)realloc(A->ar,A->nz * sizeof(double));assert(A->ar != NULL);
    }
  /* printf("[A->nz = %d  A->i = %d]",A->nz,A->i); */
  /* printf("[b->n = %d  b->i = %d]",b->n,b->i); */
  printf("%d Unleichungen fuer %d Unbekannte berechnet (%d Nicht-Null-Elemente)\n",A->n,A->m,A->nz);
}
