/*
TrussTopologyRobustRedundant - erstelle_lp

Author: Daniel Mohr
Date: 2012-02-29
*/

#include "ttrr_erstelle_lp.h"

void erstelle_lp_intern(glp_prob *lp,const char *name,const struct sparse_matrix *A,const struct double_vektor *b,const struct sparse_matrix *Aeq,const struct double_vektor *beq,const struct double_vektor *lb,const struct double_vektor *ub,const struct double_vektor *z)
{
  int i;
  int n,m,nz, *ia, *ja;
  double *ar;
  glp_set_prob_name(lp,name);
  glp_set_obj_dir(lp,GLP_MIN);
  /* A x <= b */
  glp_add_rows(lp,b->n);
  for(i = 0; i < b->n; i++)
    glp_set_row_bnds(lp,i+1,GLP_UP,0.0,b->val[i]);
  /* Aeq x = beq */
  glp_add_rows(lp,beq->n);
  for(i = 0; i < beq->n; i++)
    glp_set_row_bnds(lp,b->n+i+1,GLP_FX,beq->val[i],beq->val[i]);
  /* lb <= x <= ub */
  glp_add_cols(lp,lb->n);
  for(i = 0; i < lb->n; i++)
    glp_set_col_bnds(lp,i+1,GLP_DB,lb->val[i],ub->val[i]);
  /* Zielfunktion */
  for(i = 0; i < z->n; i++)
    glp_set_obj_coef(lp,i+1,z->val[i]);
  /* Matrix (A;Aeq) */
  n = A->n + Aeq->n;
  m = A->m;
  nz = A->nz + Aeq->nz;
  printf("erstelle Gesamtmatrix (%dx%d) aus %d Nicht-Null-Elementen...",n,m,nz);fflush(NULL);
  ia = (int*)calloc(1+nz,sizeof(int));assert(ia != NULL);
  ja = (int*)calloc(1+nz,sizeof(int));assert(ja != NULL);
  ar = (double*)calloc(1+nz,sizeof(double));assert(ar != NULL);
  for(i = 0; i < A->nz; i++)
    {
      ia[1+i] = A->ia[i]+1;
      if (ia[1+i] > A->n)
	printf("ia[%d] = %d passt nicht (Ungleichung A)\n",1+i,ia[1+i]);
      ja[1+i] = A->ja[i]+1;
      if (ja[1+i] > A->m)
	printf("ja[%d] = %d passt nicht (Ungleichung A)\n",1+i,ja[1+i]);
      ar[1+i] = A->ar[i];
    }
  for(i = 0; i < Aeq->nz; i++)
    {
      ia[1+A->nz+i] = A->n+Aeq->ia[i]+1;
      if (ia[1+A->nz+i] > n)
	printf("ia[%d] = %d passt nicht (Gleichung Aeq)\n",1+A->nz+i,ia[1+A->nz+i]);
      ja[1+A->nz+i] = Aeq->ja[i]+1;
      if (ja[1+A->nz+i] > m)
	printf("ja[%d] = %d passt nicht (Gleichung Aeq)\n",1+A->nz+i,ja[1+A->nz+i]);
      ar[1+A->nz+i] = Aeq->ar[i];
    }
  printf("erstellt; lade Gesamtmatrix...");fflush(NULL);
  glp_load_matrix(lp,nz,ia,ja,ar);
  printf("Gesamtmatrix geladen\n");
}

void erstelle_lp(glp_prob *lp,const char *name,const struct sparse_matrix *A,const struct double_vektor *b,const struct sparse_matrix *Aeq,const struct double_vektor *beq,const struct double_vektor *lb,const struct double_vektor *ub,const struct double_vektor *z)
{
  erstelle_lp_intern(lp,name,A,b,Aeq,beq,lb,ub,z);
}

void erstelle_lpmip(glp_prob *lpmip,const char *name,const struct sparse_matrix *A,const struct double_vektor *b,const struct sparse_matrix *Aeq,const struct double_vektor *beq,const struct double_vektor *lb,const struct double_vektor *ub,const struct double_vektor *z,const int redundanz,const struct int_matrix *staebe)
{
  int i;
  erstelle_lp_intern(lpmip,name,A,b,Aeq,beq,lb,ub,z);
  for(i = 0; i < abs(redundanz) * staebe->n; i++)
    glp_set_col_kind(lpmip,i+1,GLP_BV);
}
