/*
TrussTopologyRobustRedundant - lesen

Author: Daniel Mohr
Date: 2012-02-26
*/

#include "ttrr_lesen.h"

void lese_int(int *variable,FILE *fd)
{
  /* lese int aus einer geoeffneten Datei */
  if (fread(variable,sizeof(*variable),1,fd)!=1)
    {
      fclose(fd);
      fprintf(stderr,"I/O-error\n");
      exit(EXIT_FAILURE);
    }
}

void lese_double(double *variable,FILE *fd)
{
  /* lese double aus einer geoeffneten Datei */
  if (fread(variable,sizeof(*variable),1,fd)!=1)
    {
      fclose(fd);
      fprintf(stderr,"I/O-error\n");
      exit(EXIT_FAILURE);
    }
}

void lese_int_from(const char *pathname,const char *filename,int *variable)
{
  /* lese int aus der Datei pathname/filename.bin oder pathname/filename.txt */
  FILE *fd;
  char *fnbin, *fntxt;
  pathnamefilename(&fnbin,pathname,filename,".bin");
  pathnamefilename(&fntxt,pathname,filename,".txt");
  fd = fopen(fnbin,"rb");
  if (fd)
    {
      printf("lese %s\n",fnbin);
      if (fread(variable,sizeof(*variable),1,fd)!=1)
	{
	  fclose(fd);
	  fprintf(stderr,"I/O-error\n");
	  exit(EXIT_FAILURE);
	}
      fclose(fd);
    } else {
    fd = fopen(fntxt,"r");
    if( !(fd) )
      {
	fprintf(stderr,"I/O-error\nFile '%s' kann nicht gelesen werden.\n",fntxt);
	exit(EXIT_FAILURE);
      }
    printf("lese %s\n",fntxt);
    if (!fscanf(fd, "%d",variable)){fprintf(stderr,"I/O-error\nFile");exit(EXIT_FAILURE);}
    fclose(fd);
  }
  free(fnbin);
  free(fntxt);
}

void lese_double_from(const char *pathname,const char *filename,double *variable)
{
  /* lese double aus der Datei pathname/filename.bin oder pathname/filename.txt */
  FILE *fd;
  char *fnbin, *fntxt;
  pathnamefilename(&fnbin,pathname,filename,".bin");
  pathnamefilename(&fntxt,pathname,filename,".txt");
  fd = fopen(fnbin,"rb");
  if (fd)
    {
      printf("lese %s\n",fnbin);
      if (fread(variable,sizeof(*variable),1,fd)!=1)
	{
	  fclose(fd);
	  fprintf(stderr,"I/O-error\n");
	  exit(EXIT_FAILURE);
	}
      fclose(fd);
    } else {
    fd = fopen(fntxt,"r");
    if( !(fd) )
      {
	fprintf(stderr,"I/O-error\nFile '%s' kann nicht gelesen werden.\n",fntxt);
	exit(EXIT_FAILURE);
      }
    printf("lese %s\n",fntxt);
    if (!fscanf(fd, "%le",variable)){fprintf(stderr,"I/O-error\nFile");exit(EXIT_FAILURE);}
    fclose(fd);
  }
  free(fnbin);
  free(fntxt);
}

void lese_matrix_int(const char *pathname,const char *filename,struct int_matrix *A)
{ /* lese Matrix aus der Datei pathname/filename.bin oder pathname/filename.txt */
  /* struct int_matrix{int n,m;int **val;}; */
  FILE *fd;
  char *fnbin, *fntxt;
  int i,j;
  pathnamefilename(&fnbin,pathname,filename,".bin");
  pathnamefilename(&fntxt,pathname,filename,".txt");
  fd = fopen(fnbin,"rb");
  if (fd)
    {
      printf("lese %s\n",fnbin);
      lese_int(&A->n,fd);
      lese_int(&A->m,fd);
      A->val = (int**)calloc(A->n,sizeof(int*));assert(A->val != NULL);
      A->val[0] = (int*)calloc(A->m*A->n,sizeof(int));assert(A->val[0] != NULL);
      for(i=1; i<A->n; i++)
	A->val[i] = A->val[0] + i*A->m;
      /* A->val[0][0] bis A->val[n-1][m-1] stehen jetzt zur Verfuegung */
      for(i=0; i<A->n; i++)
	for(j=0; j<A->m; j++)
	    lese_int(&A->val[i][j],fd);
      fclose(fd);
    } else {
    fd = fopen(fntxt,"r");
    if( !(fd) )
      {
	fprintf(stderr,"I/O-error\nFile '%s' kann nicht gelesen werden.\n",fntxt);
	exit(EXIT_FAILURE);
      }
    printf("lese %s\n",fntxt);
    if (!fscanf(fd, "%d %d",&A->n,&A->m)){fprintf(stderr,"I/O-error\nFile");exit(EXIT_FAILURE);}
    A->val = (int**)calloc(A->n,sizeof(int*));assert(A->val != NULL);
    A->val[0] = (int*)calloc(A->m*A->n,sizeof(int));assert(A->val[0] != NULL);
    for(i=1; i<A->n; i++)
      A->val[i] = A->val[0] + i*A->m;
    /* A->val[0][0] bis A->val[n-1][m-1] stehen jetzt zur Verfuegung */
    for(i=0; i<A->n; i++)
      {
	if (A->m == 2)
	  if (!fscanf(fd, "%d %d ",&A->val[i][0],&A->val[i][1])){fprintf(stderr,"I/O-error\nFile");exit(EXIT_FAILURE);}
	if (A->m == 3)
	  if (!fscanf(fd, "%d %d %d ",&A->val[i][0],&A->val[i][1],&A->val[i][2])){fprintf(stderr,"I/O-error\nFile");exit(EXIT_FAILURE);}
      }
    fclose(fd);
  }
}

void lese_matrix_double(const char *pathname,const char *filename,struct double_matrix *A)
{ /* lese Matrix aus der Datei pathname/filename.bin oder pathname/filename.txt */
  /* struct double_matrix{int n,m;double **val;}; */
  FILE *fd;
  char *fnbin, *fntxt;
  int i,j;
  pathnamefilename(&fnbin,pathname,filename,".bin");
  pathnamefilename(&fntxt,pathname,filename,".txt");
  fd = fopen(fnbin,"rb");
  if (fd)
    {
      printf("lese %s\n",fnbin);
      lese_int(&A->n,fd);
      lese_int(&A->m,fd);
      A->val = (double**)calloc(A->n,sizeof(double*));assert(A->val != NULL);
      A->val[0] = (double*)calloc(A->m*A->n,sizeof(double));assert(A->val[0] != NULL);
      for(i=1; i<A->n; i++)
	A->val[i] = A->val[0] + i*A->m;
      /* A->val[0][0] bis A->val[n-1][m-1] stehen jetzt zur Verfuegung */
      for(i=0; i<A->n; i++)
	for(j=0; j<A->m; j++)
	    lese_double(&A->val[i][j],fd);
      fclose(fd);
    } else {
    fd = fopen(fntxt,"r");
    if( !(fd) )
      {
	fprintf(stderr,"I/O-error\nFile '%s' kann nicht gelesen werden.\n",fntxt);
	exit(EXIT_FAILURE);
      }
    printf("lese %s\n",fntxt);
    if (!fscanf(fd, "%d %d",&A->n,&A->m)){fprintf(stderr,"I/O-error\nFile");exit(EXIT_FAILURE);}
    A->val = (double**)calloc(A->n,sizeof(double*));assert(A->val != NULL);
    A->val[0] = (double*)calloc(A->m*A->n,sizeof(double));assert(A->val[0] != NULL);
    for(i=1; i<A->n; i++)
      A->val[i] = A->val[0] + i*A->m;
    /* A->val[0][0] bis A->val[n-1][m-1] stehen jetzt zur Verfuegung */
    for(i=0; i<A->n; i++)
      {
	if (A->m == 2)
	  if (!fscanf(fd, "%le %le ",&A->val[i][0],&A->val[i][1])){fprintf(stderr,"I/O-error\nFile");exit(EXIT_FAILURE);}
	if (A->m == 3)
	  if (!fscanf(fd, "%le %le %le ",&A->val[i][0],&A->val[i][1],&A->val[i][2])){fprintf(stderr,"I/O-error\nFile");exit(EXIT_FAILURE);}
      }
    fclose(fd);
  }
}

void lese_vektor(const char *pathname,const char *filename,struct int_vektor *b)
{
  /* lese Matrix aus der Datei pathname/filename.bin oder pathname/filename.txt */
  /* struct int_vektor{int n;int *val;}; */
  FILE *fd;
  char *fnbin, *fntxt;
  int i;
  pathnamefilename(&fnbin,pathname,filename,".bin");
  pathnamefilename(&fntxt,pathname,filename,".txt");
  fd = fopen(fnbin,"rb");
  if (fd)
    {
      printf("lese %s\n",fnbin);
      lese_int(&b->n,fd);
      b->val = (int*)calloc(b->n,sizeof(int));assert(b->val != NULL);
      for(i=0; i<b->n; i++)
	lese_int(&b->val[i],fd);
      fclose(fd);
    } else {
    fd = fopen(fntxt,"r");
    if( !(fd) )
      {
	fprintf(stderr,"I/O-error\nFile '%s' kann nicht gelesen werden.\n",fntxt);
	exit(EXIT_FAILURE);
      }
    printf("lese %s\n",fntxt);
    if (!fscanf(fd, "%d",&b->n)){fprintf(stderr,"I/O-error\nFile");exit(EXIT_FAILURE);}
      b->val = (int*)calloc(b->n,sizeof(int));assert(b->val != NULL);
    for(i=0; i<b->n; i++)
      if (!fscanf(fd, "%d",&b->val[i])){fprintf(stderr,"I/O-error\nFile");exit(EXIT_FAILURE);}
    fclose(fd);
  }
}

void lese_sparse_matrix(const char *pathname,const char *filename,struct sparse_matrix *kraefte)
{
  /* lese Sparse-Matrix aus der Datei pathname/filename.bin oder pathname/filename.txt */
  FILE *fd;
  char *fnbin, *fntxt;
  int i;
  /* struct sparse_matrix{int i,n,m,nz,*ia,*ja;double *ar;}; */
  pathnamefilename(&fnbin,pathname,filename,".bin");
  pathnamefilename(&fntxt,pathname,filename,".txt");
  fd = fopen(fnbin,"rb");
  if (fd)
    {
      printf("lese %s\n",fnbin);
      lese_int(&kraefte->n,fd);
      lese_int(&kraefte->m,fd);
      lese_int(&(*kraefte).nz,fd);
      kraefte->ia = (int*)calloc(kraefte->nz,sizeof(int));assert(kraefte->ia != NULL);
      kraefte->ja = (int*)calloc(kraefte->nz,sizeof(int));assert(kraefte->ja != NULL);
      kraefte->ar = (double*)calloc(kraefte->nz,sizeof(double));assert(kraefte->ar != NULL);
      for(i = 0; i < kraefte->nz; i++)
	lese_int(&kraefte->ia[i],fd);
      for(i = 0; i < kraefte->nz; i++)
	lese_int(&kraefte->ja[i],fd);
      for(i = 0; i < kraefte->nz; i++)
	lese_double(&kraefte->ar[i],fd);
      fclose(fd);
    } else {
    fd = fopen(fntxt,"r");
    if( !(fd) )
      {
	fprintf(stderr,"I/O-error\nFile '%s' kann nicht gelesen werden.\n",fntxt);
	exit(EXIT_FAILURE);
      }
    printf("lese %s\n",fntxt);
    if (!fscanf(fd, "%d %d %d",&kraefte->n,&kraefte->m,&kraefte->nz)){fprintf(stderr,"I/O-error\nFile");exit(EXIT_FAILURE);}
    kraefte->ia = (int*)calloc(kraefte->nz,sizeof(int));assert(kraefte->ia != NULL);
    kraefte->ja = (int*)calloc(kraefte->nz,sizeof(int));assert(kraefte->ja != NULL);
    kraefte->ar = (double*)calloc(kraefte->nz,sizeof(double));assert(kraefte->ar != NULL);
    for(i = 0; i < kraefte->nz; i++)
      if (!fscanf(fd, "%d %d %le",&kraefte->ia[i],&kraefte->ja[i],&kraefte->ar[i])){fprintf(stderr,"I/O-error\nFile");exit(EXIT_FAILURE);}
    fclose(fd);
  }
}

void lese_string(const char *pathname,const char *filename,char **variable,int l)
{
  char *fntxt;
  FILE *fd;
  pathnamefilename(&fntxt,pathname,filename,"");
  fd = fopen(fntxt,"r");
    if( !(fd) )
      {
	fprintf(stderr,"I/O-error\nFile '%s' kann nicht gelesen werden.\n",fntxt);
	exit(EXIT_FAILURE);
      }
  printf("lese %s\n",fntxt);
  *variable = (char*)malloc((l+1)*sizeof(char));assert(variable != NULL);
  *variable=fgets(*variable,l+1,fd);
  fclose(fd);
}
