/*
TrussTopologyRobustRedundant - scale

Author: Daniel Mohr
Date: 2012-02-29

*/

#include "ttrr_scale.h"

void scale(double *scale_sigma,double *scale_kraefte,struct sparse_matrix *kraefte,double *sigmamin,double *sigmamax,double *smax)
{
  int i;
  double m;
  if ((*scale_sigma == 0) && (*scale_kraefte == 0))
    { /* auto scaling */
      m = max(abs(*sigmamin),abs(*sigmamax));
      for(i = 0; i < kraefte->nz; i++)
	{
	  m = max(m,abs(kraefte->ar[i]));
	}
      *scale_sigma = 1.0 / m;
      *scale_kraefte = 1.0 / m;
    } else {
    if (*scale_sigma == 0)
      { /* auto scaling */
	m = max(abs(*sigmamin),abs(*sigmamax));
	*scale_sigma = 1.0 / m;
      } else {
      if (*scale_kraefte == 0)
	{ /* auto scaling */
	  m = 0;
	  for(i = 0; i < kraefte->nz; i++)
		     {
		       m = max(m,abs(kraefte->ar[i]));
		     }
	  *scale_kraefte = 1.0 / m;
	}
    }}
  if (*scale_sigma > 0)
    { /* scale sigmamin, sigmamax, s, smax */
      printf("scale sigmamin, sigmamax, smax with %e\n",*scale_sigma);
      *sigmamin *= *scale_sigma;
      *sigmamax *= *scale_sigma;
      *smax /= *scale_sigma;
    }
  if (*scale_kraefte > 0)
    { /* scale kraefte, w, s, smax */
      printf("scale kraefte, smax with %e\n",*scale_kraefte);
      for(i = 0; i < kraefte->nz; i++)
	{
	  kraefte->ar[i] *= *scale_kraefte;
	}
      *smax *= *scale_kraefte;
    }
}

void unscale(double *scale_sigma,double *scale_kraefte,struct sparse_matrix *kraefte,double *sigmamin,double *sigmamax,double *smax,struct double_vektor *w,struct double_vektor *s,struct double_vektor *solution)
{
  int i;
  if (*scale_sigma > 0)
    { /* scale sigmamin, sigmamax, s, smax */
      printf("unscale sigmamin, sigmamax, s, smax with %e\n",*scale_sigma);
      *sigmamin /= *scale_sigma;
      *sigmamax /= *scale_sigma;
      *smax *= *scale_sigma;
      for(i = 0; i < s->n; i++)
	{
	  s->val[i] *= *scale_sigma;
	}
      for(i = 0; i < solution->n; i++)
	{
	  if (solution->n - s->n <= i)
	    solution->val[i] *= *scale_sigma; /* s */
	}
    }
  if (*scale_kraefte > 0)
    { /* scale kraefte, w, s, smax */
      printf("unscale kraefte, w, s, smax with %e\n",*scale_kraefte);
      for(i = 0; i < kraefte->nz; i++)
	{
	  kraefte->ar[i] /= *scale_kraefte;
	}
      *smax /= *scale_kraefte;
      for(i = 0; i < w->n; i++)
	{
	  w->val[i] /= *scale_kraefte;
	}
      for(i = 0; i < s->n; i++)
	{
	  s->val[i] /= *scale_kraefte;
	}
      for(i = 0; i < solution->n; i++)
	{
	  if ((solution->n - s->n - w->n <= i) && (i < solution->n - s->n))
	    solution->val[i] /= *scale_kraefte; /* w */
	  else if (solution->n - s->n <= i)
	    solution->val[i] /= *scale_kraefte; /* s */
	}
    }
}
