/*
TrussTopologyRobustRedundant - schreiben

Author: Daniel Mohr
Date: 2012-02-26
*/

#include "ttrr_schreiben.h"

void schreibe_int(int *variable,FILE *fd)
{
  /* schreibe int in eine geoeffnete Datei */
  if (fwrite(variable,sizeof(*variable),1,fd)!=1)
    {
      fclose(fd);
      fprintf(stderr,"I/O-error: could not write int\n");
      exit(EXIT_FAILURE);
    }
}
void schreibe_double(double *variable,FILE *fd)
{
  /* schreibe double in eine geoeffnete Datei */
  if (fwrite(variable,sizeof(*variable),1,fd)!=1)
    {
      fclose(fd);
      fprintf(stderr,"I/O-error: could not write double\n");
      exit(EXIT_FAILURE);
    }
}

void schreibe_double_vektor(const char *pathname,const char *filename,struct double_vektor *b)
{
  /* schreibe Vektor in die Datei pathname/filename.bin und pathname/filename.txt */
  /* struct double_vektor{int n;int *val;}; */
  FILE *fd;
  char *fnbin, *fntxt;
  int i;
  pathnamefilename(&fnbin,pathname,filename,".bin");
  pathnamefilename(&fntxt,pathname,filename,".txt");
  fd = fopen(fnbin,"wb");
  printf("schreibe %s\n",fnbin);
  schreibe_int(&b->n,fd);
  for(i=0; i<b->n; i++)
    schreibe_double(&b->val[i],fd);
  fclose(fd);
  fd = fopen(fntxt,"w");
  printf("schreibe %s\n",fntxt);
  fprintf(fd,"%d\n",b->n);
  for(i=0; i<b->n; i++)
    fprintf(fd,"%e\n",b->val[i]);
  fclose(fd);
}

void schreibe_int_vektor(const char *pathname,const char *filename,struct int_vektor *b)
{
  /* schreibe Vektor in die Datei pathname/filename.bin und pathname/filename.txt */
  /* struct int_vektor{int n;int *val;}; */
  FILE *fd;
  char *fnbin, *fntxt;
  int i;
  pathnamefilename(&fnbin,pathname,filename,".bin");
  pathnamefilename(&fntxt,pathname,filename,".txt");
  fd = fopen(fnbin,"wb");
  printf("schreibe %s\n",fnbin);
  schreibe_int(&b->n,fd);
  for(i=0; i<b->n; i++)
    schreibe_int(&b->val[i],fd);
  fclose(fd);
  fd = fopen(fntxt,"w");
  printf("schreibe %s\n",fntxt);
  fprintf(fd,"%d\n",b->n);
  for(i=0; i<b->n; i++)
    fprintf(fd,"%d\n",b->val[i]);
  fclose(fd);
}

void schreibe_sparse_matrix(const char *pathname,const char *filename,struct sparse_matrix *A)
{
  /* lese Sparse-Matrix aus der Datei pathname/filename.bin oder pathname/filename.txt */
  FILE *fd;
  char *fnbin, *fntxt;
  int i;
  /* struct sparse_matrix{int i,n,m,nz,*ia,*ja;double *ar;}; */
  pathnamefilename(&fnbin,pathname,filename,".bin");
  pathnamefilename(&fntxt,pathname,filename,".txt");
  fd = fopen(fnbin,"wb");
  printf("schreibe %s\n",fnbin);
  schreibe_int(&A->n,fd);
  schreibe_int(&A->m,fd);
  schreibe_int(&A->nz,fd);
  for(i=0; i<A->nz; i++)
    schreibe_int(&A->ia[i],fd);
  for(i=0; i<A->nz; i++)
    schreibe_int(&A->ja[i],fd);
  for(i=0; i<A->nz; i++)
    schreibe_double(&A->ar[i],fd);
  fclose(fd);
  fd = fopen(fntxt,"w");
  printf("schreibe %s\n",fntxt);
  fprintf(fd,"%d %d %d\n",A->n,A->m,A->nz);
  for(i=0; i<A->nz; i++)
    fprintf(fd,"%d %d %e\n",A->ia[i],A->ja[i],A->ar[i]);
  fclose(fd);
}

void schreibe_lp(const char *pathname,glp_prob *lp,const char *name)
{
  /* schreibe Vektor in die Datei pathname/filename.bin und pathname/filename.txt */
  /* struct double_vektor{int n;int *val;}; */
  char *filename1, *filename2;
  pathnamefilename(&filename1,pathname,name,".glpk.gz");
  pathnamefilename(&filename2,pathname,name,".prob.gz");
  glp_write_lp(lp,NULL,filename1);
  glp_write_prob(lp,0,filename2);
}

void schreibe_lp_solution(const char *pathname,glp_prob *lp,const char *name,const int redundanz, const int opt)
{
  /* schreibe Vektor in die Datei pathname/filename.bin und pathname/filename.txt */
  /* struct double_vektor{int n;int *val;}; */
  char *filename1, *filename2;
  pathnamefilename(&filename1,pathname,name,".out.gz");
  pathnamefilename(&filename2,pathname,name,".sol.gz");
  if (redundanz == 0)
    {
      if (opt == 0)
	{
	  /* opt == 0: glp_simplex */
	  glp_print_sol(lp,filename1);
	  glp_write_sol(lp,filename2);
	} else {
	/* opt == 1: glp_interior */
	glp_print_ipt(lp,filename1);
	glp_write_ipt(lp,filename2);
      }
    } else {
    /* glp_intopt */
      glp_print_mip(lp,filename1);
      glp_write_mip(lp,filename2);
  }
}

void schreibe_double_file(const char *pathname,const char *filename,double *wert)
{
  /* schreibe wert in die Datei pathname/filename.bin und pathname/filename.txt */
  FILE *fd;
  char *fnbin, *fntxt;
  pathnamefilename(&fnbin,pathname,filename,".bin");
  pathnamefilename(&fntxt,pathname,filename,".txt");
  fd = fopen(fnbin,"wb");
  printf("schreibe %s\n",fnbin);
  schreibe_double(wert,fd);
  fclose(fd);
  fd = fopen(fntxt,"w");
  printf("schreibe %s\n",fntxt);
  fprintf(fd,"%e\n",*wert);
  fclose(fd);
}
