#!/usr/bin/python -BO
#!/usr/bin/python -O
import ttrr as top
import numpy,scipy.sparse
# definition of the design space
dim = 3 # dimension of the design space
nelx = 3 # number of nodal points in x-direction
nely = nelx # number of nodal points in y-direction
nelz = nelx # number of nodal points in z-direction
# definition of the grid (in meter
n = max(nelx,nely,nelz)
gx0 = 0.0
gx1 = (nelx-1)*1.0/(n-1)
gy0 = 0.0
gy1 = (nely-1)*1.0/(n-1)
gz0 = 0.0
gz1 = (nelz-1)*1.0/(n-1)
print('gx0 = %f ; gx1 = %f' % (gx0,gx1))
print('gy0 = %f ; gy1 = %f' % (gy0,gy1))
print('gz0 = %f ; gz1 = %f' % (gz0,gz1))
##################################################
# bearing: (fixed bearings on the ground)
lagerbereiche = numpy.array([
    [ gx0 , gx1 , gy0 , gy1 , gz0 , gz0 ]
    ])
lagerbereiche[:,[0,2,4]] = lagerbereiche[:,[0,2,4]] - 1e-6
lagerbereiche[:,[1,3,5]] = lagerbereiche[:,[1,3,5]] + 1e-6
##################################################
maxstablaenge = 3 # maximum allowed length of one truss (here: all)
[knoten,staebe,lager] = top.staebeknotenaufgitter3d(gx0,gx1,gy0,gy1,gz0,gz1,nelx,nely,nelz,lagerbereiche=lagerbereiche,maxstablaenge=maxstablaenge)
(kn,kr) = knoten.shape
(sn,sr) = staebe.shape
(ln,) = lager.shape
##################################################
print('%d points and %d trusses' % (kn,sn))
ls = ''
for i in range(0,ln):
    ls = '%s %d' % (ls,lager[i])
print('fixed bearing points direction: %s' % (ls))
##################################################
# loads:
kraefte = scipy.sparse.lil_matrix((dim*kn,2),dtype=numpy.float64) # 2 load cases
k1 = top.knotennummervonkoordinaten(knoten,gx0,(gy0+gy1)/2,gz1) # position of the first force
r1 = dim*k1 # direction of the first force
kraefte[r1,0] = -100000.0 # Newton = ca. 1000 kg
k2 = top.knotennummervonkoordinaten(knoten,gx1,(gy0+gy1)/2,gz1) # position of the second force
r2 = dim*k2 # direction of the second force
kraefte[r2,1] = 100000.0 # Newton = ca. 1000 kg
print "kraefte",kraefte
print('force in %d in the direction %d (load case 1): %e' % (k1,r1,kraefte[r1,0]))
print('force in %d in the direction %d (load case 2): %d' % (k2,r2,kraefte[r2,1]))
##################################################
# calculation
smax=0.01 # Meter^2
name="kran%sx%sx%s_lastfall2_interior" % (nelx,nely,nelz)
[s,w,laengen] = top.ttrr(dim,knoten,staebe,lager,kraefte,smax,name=name,redundanz=0,opt=1)
name="kran%sx%sx%s_lastfall2_simplex" % (nelx,nely,nelz)
[s,w,laengen] = top.ttrr(dim,knoten,staebe,lager,kraefte,smax,name=name,redundanz=0,opt=0)
redundanz = 2
name="redundant_1durch%d_kran%sx%sx%s_lastfall2" % (redundanz,nelx,nely,nelz)
[s,w,laengen] = top.ttrr_redundanz(dim,knoten,staebe,lager,kraefte,smax,name=name,redundanz=redundanz,opt=4,sort=1)
name="redundant_1minus1durch%d_kran%sx%sx%s_lastfall2" % (redundanz,nelx,nely,nelz)
[s,w,laengen] = top.ttrr_redundanz(dim,knoten,staebe,lager,kraefte,smax,name=name,redundanz=-redundanz,opt=3,sort=1)
redundanz = 3
name="redundant_1durch%d_kran%sx%sx%s_lastfall2_greedy" % (redundanz,nelx,nely,nelz)
[s,w,laengen] = top.ttrr_redundanz(dim,knoten,staebe,lager,kraefte,smax,name=name,redundanz=redundanz,opt=5)
name="redundant_1durch%d_kran%sx%sx%s_lastfall2" % (redundanz,nelx,nely,nelz)
[s,w,laengen] = top.ttrr_redundanz(dim,knoten,staebe,lager,kraefte,smax,name=name,redundanz=redundanz,opt=3,sort=1)
name="redundant_1minus1durch%d_kran%sx%sx%s_lastfall2" % (redundanz,nelx,nely,nelz)
[s,w,laengen] = top.ttrr_redundanz(dim,knoten,staebe,lager,kraefte,smax,name=name,redundanz=-redundanz,opt=3,sort=1)
name="robust_smallnoise_kran%sx%sx%s_lastfall2_interior" % (nelx,nely,nelz)
[s,w,laengen] = top.ttrr_robust(dim,knoten,staebe,lager,kraefte,smax,name=name,redundanz=0,opt=1)
name="robust_smallnoise_kran%sx%sx%s_lastfall2_simplex" % (nelx,nely,nelz)
[s,w,laengen] = top.ttrr_robust(dim,knoten,staebe,lager,kraefte,smax,name=name,redundanz=0,opt=0)
name="robust_mediumnoise_kran%sx%sx%s_lastfall2_interior" % (nelx,nely,nelz)
[s,w,laengen] = top.ttrr_robust(dim,knoten,staebe,lager,kraefte,smax,name=name,redundanz=0,opt=1,p=0.5)
name="robust_mediumnoise_kran%sx%sx%s_lastfall2_simplex" % (nelx,nely,nelz)
[s,w,laengen] = top.ttrr_robust(dim,knoten,staebe,lager,kraefte,smax,name=name,redundanz=0,opt=0,p=0.5)
name="robust_bignoise_kran%sx%sx%s_lastfall2_interior" % (nelx,nely,nelz)
[s,w,laengen] = top.ttrr_robust(dim,knoten,staebe,lager,kraefte,smax,name=name,redundanz=0,opt=1,p=1)
name="robust_bignoise_kran%sx%sx%s_lastfall2_simplex" % (nelx,nely,nelz)
[s,w,laengen] = top.ttrr_robust(dim,knoten,staebe,lager,kraefte,smax,name=name,redundanz=0,opt=0,p=1)
