#!/usr/bin/python -BO
import ttrr,numpy,scipy
import argparse,tempfile,tarfile,os,shutil

def create_results(args):
    resultfile = args.resultfile
    tmpdir = tempfile.mkdtemp(prefix='ttrr_tmp_') # results
    if os.path.isfile(resultfile) and tarfile.is_tarfile(resultfile):
        tar = tarfile.open(resultfile,'r:bz2')
        tar.extractall(path=tmpdir)
        tar.close()
    for file in args.file:
        if ((not os.path.isfile(resultfile)) or
            (os.path.isfile(resultfile) and (not os.path.samefile(resultfile,file)))):
            dd = os.path.splitext(file)[0]
            dd = os.path.splitext(dd)[0]
            ddo = os.path.join(tmpdir,dd)
            if not os.path.isdir(ddo):
                os.mkdir(ddo)
            print('store infos from %s' % file)
            if os.path.isfile(file) and tarfile.is_tarfile(file):
                tar = tarfile.open(file,'r:bz2')
                tar_members = tar.getmembers()
                tar_names = tar.getnames()
                if ("s.bin" in tar_names):
                    tar.extractall(path=ddo,members=[tar_members[tar_names.index('s.bin')]])
                if ("w.bin" in tar_names):
                    tar.extractall(path=ddo,members=[tar_members[tar_names.index('w.bin')]])
                if ("C.bin" in tar_names):
                    tar.extractall(path=ddo,members=[tar_members[tar_names.index('C.bin')]])
                if ("zielfktwert.bin" in tar_names):
                    tar.extractall(path=ddo,members=[tar_members[tar_names.index('zielfktwert.bin')]])
                tar.close()
    if os.path.isfile(resultfile):
        os.unlink(resultfile)
    tar = tarfile.open(resultfile,'w:bz2')
    for d in os.listdir(tmpdir):
        tar.add('%s/%s' % (tmpdir,d),arcname=d)
    tar.close()
    shutil.rmtree(tmpdir,ignore_errors=True)
    return

def compare_result(was,dd,tarres,tarres_members,tarres_names,tar,tar_members,tar_names):
    ok = 0
    if ('s.bin' in tar_names) and (('%s/s.bin' % dd) in tarres_names):
        if tar_members[tar_names.index('s.bin')].size == tarres_members[tarres_names.index('%s/s.bin' % dd)].size:
            # dateigroesse passt
            fd = tar.extractfile('s.bin')
            a = fd.read(tar_members[tar_names.index('s.bin')].size)
            fd.close()
            fd = tarres.extractfile('%s/s.bin' % dd)
            b = fd.read(tarres_members[tarres_names.index('%s/s.bin' % dd)].size)
            fd.close()
            if a == b:
                ok = 1
            else:
                ok = -1
    return ok
def compare_results(args):
    resultfile = args.resultfile
    erg = [0,0,0]
    if os.path.isfile(resultfile) and tarfile.is_tarfile(resultfile):
        tarres = tarfile.open(resultfile,'r:bz2')
        tarres_members = tarres.getmembers()
        tarres_names = tarres.getnames()
        for file in args.file:
            if ((not os.path.isfile(resultfile)) or
                (os.path.isfile(resultfile) and (not os.path.samefile(resultfile,file)))):
                dd = os.path.splitext(file)[0]
                dd = os.path.splitext(dd)[0]
                if (dd in tarres_names) and (os.path.isfile(file) and tarfile.is_tarfile(file)):
                    print('compare results from %s' % file)
                    tar = tarfile.open(file,'r:bz2')
                    tar_members = tar.getmembers()
                    tar_names = tar.getnames()
                    s = compare_result('s.bin',dd,tarres,tarres_members,tarres_names,tar,tar_members,tar_names)
                    w = compare_result('w.bin',dd,tarres,tarres_members,tarres_names,tar,tar_members,tar_names)
                    C = compare_result('C.bin',dd,tarres,tarres_members,tarres_names,tar,tar_members,tar_names)
                    zielfktwert = compare_result('zielfktwert.bin',dd,tarres,tarres_members,tarres_names,tar,tar_members,tar_names)
                    print('s: %d   w: %d   C: %d   zielfktwert: %d' % (s,w,C,zielfktwert))
                    erg[s+1] = erg[s+1] + 1
                    erg[w+1] = erg[w+1] + 1
                    erg[C+1] = erg[C+1] + 1
                    erg[zielfktwert+1] = erg[zielfktwert+1] + 1
                    tar.close()
        tarres.close()
    print('######')
    print('# error: %d' % erg[0])
    print('# compare not possible: %d' % erg[1])
    print('# OK: %d' % erg[2])
    print('######')
    return

def main():
    parser = argparse.ArgumentParser(
        description='ttrr - TrussTopologyRobustRedundant: compareresults.py',
        epilog="Author: Daniel Mohr\nDate: 2012-02-27\nLicense: GNU GENERAL PUBLIC LICENSE, Version 3, 29 June 2007",
        formatter_class=argparse.RawDescriptionHelpFormatter)
    parser.add_argument('--version',
                        action='version',
                        version='%(prog)s 2012-02-27')
    parser.add_argument('--create_results',
                        action='store_true',
                        default=False,
                        dest='create_results',
                        help='create resfile from files')
    parser.add_argument('--compare_results',
                        action='store_true',
                        default=False,
                        dest='compare_results',
                        help='compare resfile with files')
    parser.add_argument('-resultfile',
                        nargs=1,
                        type=str,
                        required=False,
                        dest='resultfile',
                        help='set resultfile as resfile. default: results.tar.bz2',
                        default='results.tar.bz2',
                        metavar='resfile')
    parser.add_argument('-f',
                        nargs='+',
                        type=str,
                        required=True,
                        dest='file',
                        help='file(s) to process',
                        metavar='file')
    args = parser.parse_args()
    if args.create_results:
        create_results(args)
    if args.compare_results:
        compare_results(args)

if __name__ == "__main__":
    main()
