from distutils.command.build_scripts import build_scripts as _build_scripts
from distutils.core import setup

class build_my_scripts(_build_scripts):
    """specialized source builder for scripts"""
    def initialize_options(self):
        self.scripts = None
        self.compiler = None
        self.verbose = None
        self.dry_run = None
        self.force = None
    def finalize_options(self):
        if not self.scripts:
            self.scripts = []
    def run(self):
        from distutils.ccompiler import new_compiler
        import distutils
        import os.path
        print('---> specialized source builder for scripts')
        self.compiler = new_compiler(compiler=self.compiler,
                                     verbose=self.verbose,
                                     dry_run=self.dry_run,
                                     force=self.force)
        distutils.sysconfig.customize_compiler(self.compiler)
        self.compiler.add_library('m')
        self.compiler.add_library('glpk')
        distutils.dir_util.mkpath('build',dry_run=self.dry_run)
        outdir = 'build/c_scripts'
        distutils.dir_util.mkpath(outdir,dry_run=self.dry_run)
        outname = self.compiler.executable_filename('ttrr_calculate')
        obj = self.compiler.compile(['c/ttrr_structs.c',
                                     'c/ttrr_schreiben_lesen.c',
                                     'c/ttrr_lesen.c',
                                     'c/ttrr_schreiben.c',
                                     'c/ttrr_tools.c',
                                     'c/ttrr_geometriematrix_stablaengen.c',
                                     'c/ttrr_berechne_freedofs.c',
                                     'c/ttrr_beschraenke_geometriematrix_auf_freedofs.c',
                                     'c/ttrr_beschraenke_kraefte_auf_freedofs.c',
                                     'c/ttrr_berechne_zielfunktion.c',
                                     'c/ttrr_berechne_gleichungen.c',
                                     'c/ttrr_berechne_ungleichungen.c',
                                     'c/ttrr_berechne_lb_ub.c',
                                     'c/ttrr_erstelle_lp.c',
                                     'c/ttrr_cb_mip_greedy.c',
                                     'c/ttrr_scale.c',
                                     'c/ttrr.c'],
                                    output_dir=outdir,
                                    extra_preargs=['-mfpmath=sse','-O2'])
        self.compiler.link_executable(obj,
                                      outname,
                                      output_dir=outdir)

setup(cmdclass={'build_c_scripts': build_my_scripts},
      name='ttrr',
      version = '2012-03-16',
      description = 'TrussTopologyRobustRedundant',
      long_description = 'ttrr is a software package to do topology optimization of a truss with respect to robustness and redundancy.',
      keywords='topology optimization truss bar robust robustness redundant redundancy mathematics visualization',
      author = 'Daniel Mohr',
      author_email = 'daniel.mohr@ttrr.cadae.de',
      url = 'http://ttrr.cadae.de/',
      download_url = 'http://ttrr.cadae.de/',
      packages = ['ttrr','ttrr.intern'],
      scripts=['ttrr_tools.py','build/c_scripts/ttrr_calculate'],
      license = 'GNU GENERAL PUBLIC LICENSE, Version 3, 29 June 2007',
      classifiers = ['Development Status :: 3 - Alpha',
                     'Environment :: Console',
                     'Intended Audience :: Developers',
                     'Intended Audience :: Science/Research',
                     'License :: OSI Approved :: GNU General Public License (GPL)',
                     'Operating System :: POSIX',
                     'Operating System :: POSIX :: BSD :: FreeBSD',
                     'Operating System :: POSIX :: BSD :: OpenBSD',
                     'Operating System :: POSIX :: Linux',
                     'Operating System :: Unix',
                     'Programming Language :: C',
                     'Programming Language :: Python',
                     'Programming Language :: Python :: 2',
                     'Programming Language :: Python :: 3',
                     'Topic :: Scientific/Engineering',
                     'Topic :: Scientific/Engineering :: Mathematics',
                     'Topic :: Scientific/Engineering :: Visualization'],
      requires = ['numpy','scipy'],
      provides = ['ttrr']
      )
