import numpy,scipy,scipy.linalg
from ttrr_tools import *

def berechne_gestoerte_kraefte(kraefte,dim,p,art=0):
    """kraefte_gestoert = berechne_gestoerte_kraefte(kraefte,dim,p,art=0)

    calculates perturbed forces from kraefte and returns them kraefte_gestoert.
    For nl load cases in kraefte you get in kraefte_gestoert (2^dim)*nl load
    cases.

    Parameters:
        dim : integer
              dimension 2 or 3 of the design space
        p : float
            percentage of magnitude of the unperturbed force is used for the
            perturbation. p*norm(kraefte[:,k]) is the magnitude of the perturbation
            of the load case k.
        art : integer
              chooses the kind of the perturbation: (optional)

              art == 0
                  generates perturbations in every direction of the original
                  forces and scales them. The maximal magnitude of the forces
                  is the same. (default)
              art == 1
                  generates perturbations in every direction of the original
                  forces. The maximal magnitude of the forces changes.

        art : integer
              waehlt die Art die gestoerrten Kraefte zu erstellen (optional)

              art == 0
                  erzeuge Stoerrung um p*norm(kraft) in jede Richtung.
                  skaliere die Kraefte so, dass die maximale Kraftgroesse
                  gleich bleibt (default)
              art == 1
                  erzeuge Stoerrung um p*norm(kraft) in jede Richtung.
                  Achtung: maximale Kraftgroesse aendert sich

    Author: Daniel Mohr.

    Date: 2012-01-23, 2012-01-28, 2012-02-27 (last change)

    License: GNU GENERAL PUBLIC LICENSE, Version 3, 29 June 2007.

    Copyright (C) 2011, 2012 Daniel Mohr
         This program is free software; you can redistribute it and/or
         modify it under the terms of the GNU General Public License as
         published by the Free Software Foundation; either version 3 of
         the License, or (at your option) any later version.

         This program is distributed in the hope that it will be useful,
         but WITHOUT ANY WARRANTY; without even the implied warranty of
         MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
         General Public License for more details.

         You should have received a copy of the GNU General Public License
         along with this program; if not, see
         http://www.gnu.org/licenses/"""
    (kkn,nl) = kraefte.shape # kkn Anzahl Knotenrichtungen; nl Anzahl Lastfaelle
    kn = int(kkn/dim) # Anzahl Knoten
    ne = 2**dim # Anzahl Ecken
    print('###### berechne_gestoerte_kraefte')
    print('(kkn,nl) = (%d,%d); Knotenzahl: %d; Dimension: %d' % (kkn,nl,kn,dim))
    if art == 0:
        kraefte_gestoert = berechne_gestoerte_kraefte(kraefte,dim,p,art=1)
        max_kraft_ungestoert = 0
        for k in range(0,nl): # fuer alle lastfaelle
            if max_kraft_ungestoert < scipy.linalg.norm(kraefte[:,k].toarray()):
                max_kraft_ungestoert = scipy.linalg.norm(kraefte[:,k].toarray())
        max_kraft_gestoert = 0
        (kkng,nlg) = kraefte.shape # kkn Anzahl Knotenrichtungen; nl Anzahl Lastfaelle
        for k in range(0,nlg): # fuer alle lastfaelle
            if max_kraft_gestoert < scipy.linalg.norm(kraefte_gestoert[:,k].toarray()):
                max_kraft_gestoert = scipy.linalg.norm(kraefte_gestoert[:,k].toarray())
        print('max_kraft_ungestoert = %f' % max_kraft_ungestoert)
        print('max_kraft_gestoert = %f' % max_kraft_gestoert)
        print('skaliere Kraefte')
        kraefte_gestoert = (max_kraft_ungestoert/max_kraft_gestoert) * kraefte_gestoert
        max_kraft_gestoert = 0
        for k in range(0,nlg): # fuer alle lastfaelle
            if max_kraft_gestoert < scipy.linalg.norm(kraefte_gestoert[:,k].toarray()):
                max_kraft_gestoert = scipy.linalg.norm(kraefte_gestoert[:,k].toarray())
        print('max_kraft_gestoert = %f' % max_kraft_gestoert)
    else:
        kraefte_gestoert = scipy.sparse.lil_matrix((dim*kn,ne*nl),dtype=numpy.float64)
        i = 0
        for k in range(0,nl): # fuer alle lastfaelle
            for e in range(0,ne): # fuer alle Ecken
                kraefte_gestoert[:,i] = kraefte[:,k]
                #kg = numpy.sqrt(dot(kraefte[:,k].transpose(),kraefte[:,k])[0,0])
                kg = scipy.linalg.norm(kraefte[:,k].toarray())
                kgg = p*kg # p % Stoerung
                ebin = dezimal2binaer(e,dim)
                for j in range(0,kn): # fuer alle Knoten
                    knotenkraft = False
                    for l in range(0,dim): # ist eine Kraft in diesem Knoten
                        if kraefte[j*dim+l,k] != 0:
                            knotenkraft = True
                    if knotenkraft:
                        for l in range(0,dim):
                            kraefte_gestoert[j*dim+l,i] = kraefte_gestoert[j*dim+l,i] + ((-1)**(1-ebin[(dim-1)-l])) * kgg
                            #kraefte_gestoert[j*dim+l,i] = kraefte_gestoert[j*dim+l,i] + ((-1)**ebin[l]) * kgg
                i = i + 1
    print('######')
    return kraefte_gestoert
