import os,tarfile,sys,tempfile
import numpy
from .ttrr_read import *

def gnuplot_pdf_header(prefix,filename):
    string = '%sset terminal pdf enhanced color size 10cm,10cm\n' % prefix
    string += '%sset output "%s"\n' % (prefix,filename)
    return string

def gnuplot_png_header(prefix,xaufloesung,yaufloesung,filename):
    #string = '%sset terminal png transparent nointerlace font "/usr/share/texmf-dist/fonts/truetype/public/gnu-freefont/FreeSans.ttf,30" enhanced size %d,%d nocrop\n' % (prefix,xaufloesung,yaufloesung)
    string = '%sset terminal png nointerlace font "/usr/share/texmf-dist/fonts/truetype/public/gnu-freefont/FreeSans.ttf,30" enhanced size %d,%d nocrop\n' % (prefix,xaufloesung,yaufloesung)
    string += '%sset output "%s"\n' % (prefix,filename)
    return string

def gnuplot_set_range_functions_2d(knoten,s,smin,smax):
    of = 0.1*(numpy.amax(knoten[:,0:1])-numpy.amin(knoten[:,0:1]))
    #of = 0.0
    string = 'set xrange [%2.1f:%2.1f]\n' % (numpy.amin(knoten[:,0])-of,numpy.amax(knoten[:,0])+of)
    string += 'set yrange [%2.1f:%2.1f]\n' % (numpy.amin(knoten[:,1])-of,numpy.amax(knoten[:,1])+of)
    string += 'set parametric\n'
    string += 'set ticslevel 0\n'
    string += 'set view equal xy\n'
    string += 'set trange [0:1]\n'
    string += 'sx(t,x0,x1) = x0 + t*(x1-x0)\n'
    string += 'sy(t,y0,y1) = y0 + t*(y1-y0)\n'
    #
    string += 'sgn1(a) = (a==0.0) ? 1.0 : sgn(a)\n'
    string += 'l(x0,x1,y0,y1) = sqrt((x1-x0)**2+(y1-y0)**2)\n'
    string += 'c(x0,x1,y0,y1) = sgn1(x1-x0) * l(x0,x1,0,0)/l(x0,x1,y0,y1) # cos(alpha)\n'
    string += 's(x0,x1,y0,y1) = sgn1(y1-y0) * l(0,0,y0,y1)/l(x0,x1,y0,y1) # sin(alpha)\n'
    # lange seiten
    string += 'sx1(t,x0,x1,y0,y1,r) = sx(t,x0-(r/2)*s(x0,x1,y0,y1),x1-(r/2)*s(x0,x1,y0,y1))\n'
    string += 'sy1(t,x0,x1,y0,y1,r) = sy(t,y0+(r/2)*c(x0,x1,y0,y1),y1+(r/2)*c(x0,x1,y0,y1))\n'
    string += 'sx2(t,x0,x1,y0,y1,r) = sx(t,x0+(r/2)*s(x0,x1,y0,y1),x1+(r/2)*s(x0,x1,y0,y1))\n'
    string += 'sy2(t,x0,x1,y0,y1,r) = sy(t,y0-(r/2)*c(x0,x1,y0,y1),y1-(r/2)*c(x0,x1,y0,y1))\n'
    # kurze Seiten
    string += 'sx3(t,x0,x1,y0,y1,r) = sx(t,x0-(r/2)*s(x0,x1,y0,y1),x0+(r/2)*s(x0,x1,y0,y1))\n'
    string += 'sy3(t,x0,x1,y0,y1,r) = sy(t,y0+(r/2)*c(x0,x1,y0,y1),y0-(r/2)*c(x0,x1,y0,y1))\n'
    string += 'sx4(t,x0,x1,y0,y1,r) = sx(t,x1-(r/2)*s(x0,x1,y0,y1),x1+(r/2)*s(x0,x1,y0,y1))\n'
    string += 'sy4(t,x0,x1,y0,y1,r) = sy(t,y1+(r/2)*c(x0,x1,y0,y1),y1-(r/2)*c(x0,x1,y0,y1))\n'
    # Diagonalen
    string += 'sx5(t,x0,x1,y0,y1,r) = sx(t,x0+(r/2)*s(x0,x1,y0,y1),x1-(r/2)*s(x0,x1,y0,y1))\n'
    string += 'sy5(t,x0,x1,y0,y1,r) = sy(t,y0-(r/2)*c(x0,x1,y0,y1),y1+(r/2)*c(x0,x1,y0,y1))\n'
    string += 'sx6(t,x0,x1,y0,y1,r) = sx(t,x0-(r/2)*s(x0,x1,y0,y1),x1+(r/2)*s(x0,x1,y0,y1))\n'
    string += 'sy7(t,x0,x1,y0,y1,r) = sy(t,y0+(r/2)*c(x0,x1,y0,y1),y1-(r/2)*c(x0,x1,y0,y1))\n'
    string += 'plot '
    return string

def gnuplot_line(knoten,staebe,r,sn,i,smin,smax,s):
    from .ttrr_visualisation import farben,skaliere
    #string = 'sx(t,%e,%e),' % (knoten[staebe[i,0],0],knoten[staebe[i,1],0])
    #string += 'sy(t,%e,%e) ' % (knoten[staebe[i,0],1],knoten[staebe[i,1],1])
    #string += 'notitle with lines lt 1 lc rgb "%s" lw 1' % (farben(r+1,1))
    #string += ' , '
    string = 'sx1(t,%e,%e,%e,%e,%e),' % (knoten[staebe[i,0],0],knoten[staebe[i,1],0],knoten[staebe[i,0],1],knoten[staebe[i,1],1],skaliere(s[r*sn+i],smin,smax,typ=3))
    string += 'sy1(t,%e,%e,%e,%e,%e) ' % (knoten[staebe[i,0],0],knoten[staebe[i,1],0],knoten[staebe[i,0],1],knoten[staebe[i,1],1],skaliere(s[r*sn+i],smin,smax,typ=3))
    string += 'notitle with lines lt 1 lc rgb "%s" lw 2' % (farben(r,1))
    string += ' , '
    string += 'sx2(t,%e,%e,%e,%e,%e),' % (knoten[staebe[i,0],0],knoten[staebe[i,1],0],knoten[staebe[i,0],1],knoten[staebe[i,1],1],skaliere(s[r*sn+i],smin,smax,typ=3))
    string += 'sy2(t,%e,%e,%e,%e,%e) ' % (knoten[staebe[i,0],0],knoten[staebe[i,1],0],knoten[staebe[i,0],1],knoten[staebe[i,1],1],skaliere(s[r*sn+i],smin,smax,typ=3))
    string += 'notitle with lines lt 1 lc rgb "%s" lw 2' % (farben(r,1))
    string += ' , '
    string += 'sx3(t,%e,%e,%e,%e,%e),' % (knoten[staebe[i,0],0],knoten[staebe[i,1],0],knoten[staebe[i,0],1],knoten[staebe[i,1],1],skaliere(s[r*sn+i],smin,smax,typ=3))
    string += 'sy3(t,%e,%e,%e,%e,%e) ' % (knoten[staebe[i,0],0],knoten[staebe[i,1],0],knoten[staebe[i,0],1],knoten[staebe[i,1],1],skaliere(s[r*sn+i],smin,smax,typ=3))
    string += 'notitle with lines lt 1 lc rgb "%s" lw 2' % (farben(r,1))
    string += ' , '
    string += 'sx4(t,%e,%e,%e,%e,%e),' % (knoten[staebe[i,0],0],knoten[staebe[i,1],0],knoten[staebe[i,0],1],knoten[staebe[i,1],1],skaliere(s[r*sn+i],smin,smax,typ=3))
    string += 'sy4(t,%e,%e,%e,%e,%e) ' % (knoten[staebe[i,0],0],knoten[staebe[i,1],0],knoten[staebe[i,0],1],knoten[staebe[i,1],1],skaliere(s[r*sn+i],smin,smax,typ=3))
    string += 'notitle with lines lt 1 lc rgb "%s" lw 2' % (farben(r,1))
    return string

def gnuplot_set_range_functions_3d(knoten,s,smin,smax):
    from .ttrr_visualisation import skaliere
    #of = 0.2*(numpy.amax(knoten)-numpy.amin(knoten))
    of = 2*skaliere(numpy.amax(s),smin,smax,typ=3)
    string = 'set xrange [%2.1f:%2.1f]\n' % (numpy.amin(knoten[:,0])-of,max(numpy.amax(knoten[:,0]),0.7*numpy.amax(knoten))+of)
    string += 'set yrange [%2.1f:%2.1f]\n' % (numpy.amin(knoten[:,1])-of,max(numpy.amax(knoten[:,1]),0.7*numpy.amax(knoten))+of)
    string += 'set zrange [%2.1f:%2.1f]\n' % (numpy.amin(knoten[:,2])-of,max(numpy.amax(knoten[:,2]),0.7*numpy.amax(knoten))+of)
    string += 'set parametric\n'
    string += 'set ticslevel 0\n'
    string += 'set view equal xyz\n'
    string += 'set hidden3d\n'
    string += 'set isosamples 15,11\n'
    string += 'set xyplane at %f\n' % numpy.amin(knoten[:,2])
    string += 'set urange [-pi:pi]\n'
    string += 'set vrange [0:1]\n'
    string += 'sgn1(a) = (a==0.0) ? 1.0 : sgn(a)\n'
    string += 'l(x0,x1,y0,y1,z0,z1) = sqrt((x1-x0)**2+(y1-y0)**2+(z1-z0)**2)\n'
    string += 'n1(x0,x1,y0,y1,z0,z1) = (y0==y1) ? 0.0 : \\\n'
    string += '                        -sgn(y1-y0)*sgn1(z1-z0)*sqrt(1/(1+(((x1-x0)**2)/((y0-y1)**2)))) # normale\n'
    string += 'n2(x0,x1,y0,y1,z0,z1) = (x0==x1) ? 0.0 : \\\n'
    string += '                        sgn(x1-x0)*sgn1(z1-z0)*sqrt(1/(1+(((y1-y0)**2)/((x0-x1)**2)))) # normale\n'
    string += 'n3(x0,x1,y0,y1) = 0\n'
    #string += 'c(x0,x1,y0,y1,z0,z1) = sgn(z1-z0)* \\\n'
    #string += '                       sqrt(l(x0,x1,y0,y1,z0,z1)**2-l(x0,x1,y0,y1,0,0)**2)/l(x0,x1,y0,y1,z0,z1) # cos(alpha)\n'
    string += 'c(x0,x1,y0,y1,z0,z1) = 1* \\\n'
    string += '                       l(0,0,0,0,z0,z1)/l(x0,x1,y0,y1,z0,z1) # cos(alpha)\n'
    string += 's(x0,x1,y0,y1,z0,z1) = 1* \\\n'
    string += '                       l(x0,x1,y0,y1,0,0)/l(x0,x1,y0,y1,z0,z1) # sin(alpha)\n'
    string += 'sx(u,v,x0,x1,y0,y1,z0,z1,r) = x0 + \\\n'
    string += '  ( n1(x0,x1,y0,y1,z0,z1)**2 * (1-c(x0,x1,y0,y1,z0,z1)) \\\n'
    string += '    + c(x0,x1,y0,y1,z0,z1) ) * r * cos(u) + \\\n'
    string += '  ( n1(x0,x1,y0,y1,z0,z1)*n2(x0,x1,y0,y1,z0,z1)*(1-c(x0,x1,y0,y1,z0,z1)) \\\n'
    string += '    -n3(x0,x1,y0,y1)*s(x0,x1,y0,y1,z0,z1) ) * r * sin(u) + \\\n'
    string += '  ( n1(x0,x1,y0,y1,z0,z1)*n3(x0,x1,y0,y1)*(1-c(x0,x1,y0,y1,z0,z1)) \\\n'
    string += '    +n2(x0,x1,y0,y1,z0,z1)*s(x0,x1,y0,y1,z0,z1) ) * l(x0,x1,y0,y1,z0,z1) * v\n'
    string += 'sy(u,v,x0,x1,y0,y1,z0,z1,r) = y0 + \\\n'
    string += '  ( n1(x0,x1,y0,y1,z0,z1)*n2(x0,x1,y0,y1,z0,z1)*(1-c(x0,x1,y0,y1,z0,z1)) \\\n'
    string += '    + n3(x0,x1,y0,y1)*s(x0,x1,y0,y1,z0,z1) ) * r * cos(u) + \\\n'
    string += '  ( n2(x0,x1,y0,y1,z0,z1)**2*(1-c(x0,x1,y0,y1,z0,z1)) \\\n'
    string += '    + c(x0,x1,y0,y1,z0,z1) ) * r * sin(u) + \\\n'
    string += '  ( n2(x0,x1,y0,y1,z0,z1)*n3(x0,x1,y0,y1)*(1-c(x0,x1,y0,y1,z0,z1)) \\\n'
    string += '    - n1(x0,x1,y0,y1,z0,z1)*s(x0,x1,y0,y1,z0,z1) ) * l(x0,x1,y0,y1,z0,z1) * v\n'
    string += 'sz(u,v,x0,x1,y0,y1,z0,z1,r) = z0 + \\\n'
    string += '  ( n1(x0,x1,y0,y1,z0,z1)*n3(x0,x1,y0,y1)*(1-c(x0,x1,y0,y1,z0,z1)) \\\n'
    string += '    - n2(x0,x1,y0,y1,z0,z1)*s(x0,x1,y0,y1,z0,z1) ) * r * cos(u) + \\\n'
    string += '  ( n2(x0,x1,y0,y1,z0,z1)*n3(x0,x1,y0,y1)*(1-c(x0,x1,y0,y1,z0,z1)) \\\n'
    string += '    + n1(x0,x1,y0,y1,z0,z1)*s(x0,x1,y0,y1,z0,z1) ) * r * sin(u) + \\\n'
    string += '  ( n3(x0,x1,y0,y1)**2*(1-c(x0,x1,y0,y1,z0,z1)) \\\n'
    string += '    + c(x0,x1,y0,y1,z0,z1) ) * l(x0,x1,y0,y1,z0,z1) * v\n'
    string += 'splot '
    return string

def gnuplot_set_range_functions(dim,knoten,s,smin,smax):
    if dim == 2:
        string = gnuplot_set_range_functions_2d(knoten,s,smin,smax)
    if dim == 3:
        string = gnuplot_set_range_functions_3d(knoten,s,smin,smax)
    return string

def gnuplot_cylinder(knoten,staebe,r,sn,i,smin,smax,s):
    from .ttrr_visualisation import farben,skaliere
    string = 'sx(u,v,%e,%e,%e,%e,%e,%e,%e),' % (knoten[staebe[i,0],0],
                                                knoten[staebe[i,1],0],
                                                knoten[staebe[i,0],1],
                                                knoten[staebe[i,1],1],
                                                knoten[staebe[i,0],2],
                                                knoten[staebe[i,1],2],
                                                skaliere(s[r*sn+i],smin,smax,typ=3))
    string += 'sy(u,v,%e,%e,%e,%e,%e,%e,%e),' % (knoten[staebe[i,0],0],
                                                 knoten[staebe[i,1],0],
                                                 knoten[staebe[i,0],1],
                                                 knoten[staebe[i,1],1],
                                                 knoten[staebe[i,0],2],
                                                 knoten[staebe[i,1],2],
                                                 skaliere(s[r*sn+i],smin,smax,typ=3))
    string += 'sz(u,v,%e,%e,%e,%e,%e,%e,%e) ' % (knoten[staebe[i,0],0],
                                                 knoten[staebe[i,1],0],
                                                 knoten[staebe[i,0],1],
                                                 knoten[staebe[i,1],1],
                                                 knoten[staebe[i,0],2],
                                                 knoten[staebe[i,1],2],
                                                 skaliere(s[r*sn+i],smin,smax,typ=3))
    string += 'notitle with lines lt 1 lc rgb "%s"' % (farben(r,1))
    return string

def create_png_pdf_gnuplot(file,smin,smax,xaufloesung,yaufloesung,neu,gnuplot_pdf,gnuplot_png,args):
    """Author: Daniel Mohr
    Date: 2012-03-07 (last change)
    License: GNU GENERAL PUBLIC LICENSE, Version 3, 29 June 2007"""
    from .ttrr_visualisation import farben,skaliere,skalieremultiplikativ,skalierelinear
    dd = os.path.splitext(file)[0]
    dd = os.path.splitext(dd)[0]
    if args.all_bars:
        dd += '_all_bars'
    ddgnu = '%s.gnu' % (dd)
    ddpdf = '%s.pdf' % (dd)
    ddpng = '%s.png' % (dd)
    if (((gnuplot_png and (neu==False or os.path.isfile(ddpng)==False)) or
          (gnuplot_pdf and (neu==False or os.path.isfile(ddpdf)==False))) and
         os.path.isfile(file) and tarfile.is_tarfile(file)):
        tar = tarfile.open(file,'r:bz2')
        tar_names = tar.getnames();
        if ((("s.bin" in tar_names) or (("s.txt" in tar_names))) and
            (("staebe.bin" in tar_names) or (("staebe.txt" in tar_names))) and
            (("knoten.bin" in tar_names) or (("knoten.txt" in tar_names))) and
            (("kraefte.bin" in tar_names) or (("kraefte.txt" in tar_names))) and
            (("lager.bin" in tar_names) or (("lager.txt" in tar_names))) and
            (("redundanz.bin" in tar_names) or (("redundanz.txt" in tar_names))) and
            (("dim.bin" in tar_names) or (("dim.txt" in tar_names)))):
            if gnuplot_pdf and gnuplot_png:
                print('create png and pdf with gnuplot for "%s"' % file)
            elif gnuplot_pdf and gnuplot_png==False:
                print('create pdf with gnuplot for "%s"' % file)
            elif gnuplot_pdf==False and gnuplot_png:
                print('create png with gnuplot for "%s"' % file)
            sys.stdout.flush()
            s = lese_double_vektor_tar(tar,tar_names,"s")
            if args.all_bars:
                for i in range(0,len(s)):
                    s[i] = 1
            staebe = lese_int_matrix_tar(tar,tar_names,"staebe")
            sn = len(staebe)
            knoten = lese_double_matrix_tar(tar,tar_names,"knoten")
            kraefte = lese_sparsematrix_tar(tar,tar_names,"kraefte")
            lager = lese_int_vektor_tar(tar,tar_names,"lager")
            dim = lese_int_tar(tar,tar_names,"dim")
            redundanz = abs(lese_int_tar(tar,tar_names,"redundanz"))
            if redundanz == 0:
                redundanz = 1
            tmpfile = tempfile.NamedTemporaryFile(prefix='ttrr_tmp_',delete=False)
            tmpfile.close()
            fdtmp = open(tmpfile.name,'w') # gnuplot ausgabe
            fd = open(ddgnu,'w') # gnuplot ausgabe
            fd.write('# %s\n' % dd)
            fd.write(gnuplot_pdf_header('# ',ddpdf))
            fd.write(gnuplot_png_header('# ',xaufloesung,yaufloesung,ddpng))
            if gnuplot_pdf:
                fdtmp.write(gnuplot_pdf_header('',ddpdf))
                fdtmp.write('load "%s"\n' % ddgnu)
            if gnuplot_png:
                fdtmp.write(gnuplot_png_header('',xaufloesung,yaufloesung,ddpng))
                fdtmp.write('load "%s"\n' % ddgnu)
            # zeichne
            j = 0
            fd.write(gnuplot_set_range_functions(dim,knoten,s,smin,smax))
            allererstesmal = True
            for r in range(0,redundanz):
                if redundanz > 1:
                    #fd.write('# Struktur %d\n' % (r+1))
                    ff = open("%s_x%d.gnu" % (dd,r+1),'w')
                    ff.write('# %s\n' % dd)
                    ff.write('# Struktur %d\n' % (r+1))
                    ff.write(gnuplot_pdf_header('# ','%s_x%d.pdf' % (dd,r+1)))
                    ff.write(gnuplot_png_header('# ',xaufloesung,yaufloesung,'%s_x%d.png' % (dd,r+1)))
                    ff.write(gnuplot_set_range_functions(dim,knoten,s,smin,smax))
                    if gnuplot_pdf:
                        fdtmp.write(gnuplot_pdf_header('','%s_x%d.pdf' % (dd,r+1)))
                        fdtmp.write('load "%s_x%d.gnu"\n' % (dd,r+1))
                    if gnuplot_png:
                        fdtmp.write(gnuplot_png_header('',xaufloesung,yaufloesung,'%s_x%d.png' % (dd,r+1)))
                        fdtmp.write('load "%s_x%d.gnu"\n' % (dd,r+1))
                erstesmal = True
                for i in range(0,sn):
                    if s[r*sn+i] >= smin:
                        if not allererstesmal:
                            fd.write(' , \\\n')
                        else:
                            allererstesmal = False
                        if redundanz > 1:
                            if not erstesmal:
                                ff.write(' , \\\n')
                            else:
                                erstesmal = False
                        if dim == 2:
                            fd.write(gnuplot_line(knoten,staebe,r,sn,i,smin,smax,s))
                            if redundanz > 1:
                                ff.write(gnuplot_line(knoten,staebe,r,sn,i,smin,smax,s))
                        if dim == 3:
                            fd.write(gnuplot_cylinder(knoten,staebe,r,sn,i,smin,smax,s))
                            if redundanz > 1:
                                ff.write(gnuplot_cylinder(knoten,staebe,r,sn,i,smin,smax,s))
                        j = j + 1
                if redundanz > 1:
                    ff.write('\n')
                    os.fsync(ff)
                    ff.close()
            fd.write('\n')
            os.fsync(fd)
            fd.close()
            os.fsync(fdtmp)
            fdtmp.close()
            prg = 'gnuplot %s' % (tmpfile.name)
            os.system(prg)
            os.unlink(tmpfile.name)
        tar.close()
    else:
        print('"%s" is no tar-file or pictures/movies exists' % file)
        if gnuplot_png and neu and os.path.isfile(ddpng):
            print('  picture "%s" already exists' % ddpng)
        if gnuplot_pdf and neu and os.path.isfile(ddpdf):
            print('  picture "%s" already exists' % ddpdf)
        if os.path.isfile(file) == False:
            print('  "%s" is no file' % file)
        if os.path.isfile(file) and (tarfile.is_tarfile(file)==False):
            print('  "%s" is no tar-file' % file)
        sys.stdout.flush()
    return
