import tarfile
from .ttrr_read import *
import scipy,numpy

def read_data(tar):
    """read data from tar.bz2-file
    
    Author: Daniel Mohr.

    Date: 2012-03-02 (last change).

    License: GNU GENERAL PUBLIC LICENSE, Version 3, 29 June 2007."""
    tar_names = tar.getnames();
    data = dict()
    if ("A.bin" in tar_names) or ("A.txt" in tar_names):
        data['A'] = lese_sparsematrix_tar(tar,tar_names,"A")
    if ("Aeq.bin" in tar_names) or ("Aeq.txt" in tar_names):
        data['Aeq'] = lese_sparsematrix_tar(tar,tar_names,"Aeq")
    if ("C.bin" in tar_names) or ("C.txt" in tar_names):
        data['C'] = lese_sparsematrix_tar(tar,tar_names,"C")
    if ("C_freedofs.bin" in tar_names) or ("C_freedofs.txt" in tar_names):
        data['C_freedofs'] = lese_sparsematrix_tar(tar,tar_names,"C_freedofs")
    if ("b.bin" in tar_names) or ("b.txt" in tar_names):
        data['b'] = lese_double_vektor_tar(tar,tar_names,"b")
    if ("beq.bin" in tar_names) or ("beq.txt" in tar_names):
        data['beq'] = lese_double_vektor_tar(tar,tar_names,"beq")
    if ("dichte.bin" in tar_names) or ("dichte.txt" in tar_names):
        data['dichte'] = lese_double_tar(tar,tar_names,"dichte")
    if ("dim.bin" in tar_names) or ("dim.txt" in tar_names):
        data['dim'] = lese_int_tar(tar,tar_names,"dim")
    if ("freedofs.bin" in tar_names) or ("freedofs.txt" in tar_names):
        data['freedofs'] = lese_int_vektor_tar(tar,tar_names,"freedofs")
    if ("info.txt" in tar_names):
        fd = tar.extractfile('info.txt')
        data['info'] = fd.readline()
        fd.close()
    if ("knoten.bin" in tar_names) or ("knoten.txt" in tar_names):
        data['knoten'] = lese_double_matrix_tar(tar,tar_names,"knoten")
    if ("kraefte.bin" in tar_names) or ("kraefte.txt" in tar_names):
        data['kraefte'] = lese_sparsematrix_tar(tar,tar_names,"kraefte")
    if ("kraefte_freedofs.bin" in tar_names) or ("kraefte_freedofs.txt" in tar_names):
        data['kraefte_freedofs'] = lese_sparsematrix_tar(tar,tar_names,"kraefte_freedofs")
    if ("laengen.bin" in tar_names) or ("laengen.txt" in tar_names):
        data['laengen'] = lese_double_vektor_tar(tar,tar_names,"laengen")
    if ("lager.bin" in tar_names) or ("lager.txt" in tar_names):
        data['lager'] = lese_int_vektor_tar(tar,tar_names,"lager")
    if ("lb.bin" in tar_names) or ("lb.txt" in tar_names):
        data['lb'] = lese_double_vektor_tar(tar,tar_names,"lb")
    if ("opt.bin" in tar_names) or ("opt.txt" in tar_names):
        data['opt'] = lese_int_tar(tar,tar_names,"opt")
    if ("redundanz.bin" in tar_names) or ("redundanz.txt" in tar_names):
        data['redundanz'] = lese_int_tar(tar,tar_names,"redundanz")
    if ("s.bin" in tar_names) or ("s.txt" in tar_names):
        data['s'] = numpy.asarray(lese_double_vektor_tar(tar,tar_names,"s"))
    if ("scale_kraefte.bin" in tar_names) or ("scale_kraefte.txt" in tar_names):
        data['scale_kraefte'] = lese_double_tar(tar,tar_names,"scale_kraefte")
    if ("scale_sigma.bin" in tar_names) or ("scale_sigma.txt" in tar_names):
        data['scale_sigma'] = lese_double_tar(tar,tar_names,"scale_sigma")
    if ("sigmamax.bin" in tar_names) or ("sigmamax.txt" in tar_names):
        data['sigmamax'] = lese_double_tar(tar,tar_names,"sigmamax")
    if ("sigmamin.bin" in tar_names) or ("sigmamin.txt" in tar_names):
        data['sigmamin'] = lese_double_tar(tar,tar_names,"sigmamin")
    if ("smax.bin" in tar_names) or ("smax.txt" in tar_names):
        data['smax'] = lese_double_tar(tar,tar_names,"smax")
    if ("sort.bin" in tar_names) or ("sort.txt" in tar_names):
        data['sort'] = lese_int_tar(tar,tar_names,"sort")
    if ("staebe.bin" in tar_names) or ("staebe.txt" in tar_names):
        data['staebe'] = lese_int_matrix_tar(tar,tar_names,"staebe")
    if ("ub.bin" in tar_names) or ("ub.txt" in tar_names):
        data['ub'] = lese_double_vektor_tar(tar,tar_names,"ub")
    if ("w.bin" in tar_names) or ("w.txt" in tar_names):
        data['w'] = lese_double_vektor_tar(tar,tar_names,"w")
    if ("x.bin" in tar_names) or ("x.txt" in tar_names):
        data['x'] = lese_int_vektor_tar(tar,tar_names,"x")
    if ("zielfktwert.bin" in tar_names) or ("zielfktwert.txt" in tar_names):
        data['zielfktwert'] = lese_double_tar(tar,tar_names,"zielfktwert")
    if ("zlb.bin" in tar_names) or ("zlb.txt" in tar_names):
        data['zlb'] = lese_double_tar(tar,tar_names,"zlb")
    if ("zub.bin" in tar_names) or ("zub.txt" in tar_names):
        data['zub'] = lese_double_tar(tar,tar_names,"zub")
    return data

def save_data_to_mat(filename):
    """write data to mat-file
    
    Author: Daniel Mohr.

    Date: 2012-03-02 (last change).

    License: GNU GENERAL PUBLIC LICENSE, Version 3, 29 June 2007."""
    if os.path.isfile(filename) and tarfile.is_tarfile(filename):
        matfile = os.path.splitext(filename)[0]
        matfile = os.path.splitext(matfile)[0]
        matfile = "%s.mat" % (matfile)
        tar = tarfile.open(filename,'r:bz2')
        data = read_data(tar)
        tar.close()
        print('save data to %s' % matfile)
        if scipy.version.version >= '0.9.0':
            scipy.io.savemat(matfile,data,do_compression=True,oned_as='row')
        else:
            scipy.io.savemat(matfile,data)
    return

def save_data_to_module(filename):
    """write data to {dat,dir,py}-files
    
    Author: Daniel Mohr.

    Date: 2012-03-02 (last change).

    License: GNU GENERAL PUBLIC LICENSE, Version 3, 29 June 2007."""
    if os.path.isfile(filename) and tarfile.is_tarfile(filename):
        matfile = os.path.splitext(filename)[0]
        matfile = os.path.splitext(matfile)[0]
        matfile = "%s" % (matfile)
        tar = tarfile.open(filename,'r:bz2')
        data = read_data(tar)
        tar.close()
        print('save data to %s.{dat,dir,py}' % matfile)
        scipy.io.save_as_module(matfile,data)
    return

def save_datas_to(args):
    """write data to mat-file and/or to {dat,dir,py}-files
    
    Author: Daniel Mohr.

    Date: 2012-03-02 (last change).

    License: GNU GENERAL PUBLIC LICENSE, Version 3, 29 June 2007."""
    if args.save_mat or args.save_module:
        for filename in args.file:
            if args.save_mat:
                save_data_to_mat(filename)
            if args.save_module:
                save_data_to_module(filename)
    return
