# ttrr: visualisation
# Author: Daniel Mohr
# License: GNU GENERAL PUBLIC LICENSE, Version 3, 29 June 2007

import tempfile,tarfile,os,subprocess,numpy,time
from datetime import datetime
from .ttrr_read import *
from .ttrr_create_png_pdf_gnuplot import *
from .ttrr_create_png_pdf_gnuplot_simple import *

def scan_files(files,minfaktor):
    """[smin,smax] = scan_files(files,minfaktor)
    scans files to determine minimal und maximal used cross section of bars in
    these files. Bars with a cross section lower than minfaktor are ignored. If
    minfaktor == 0: mifaktor is ignored.
    
    Author: Daniel Mohr
    Date: 2012-02-15 (last change)
    License: GNU GENERAL PUBLIC LICENSE, Version 3, 29 June 2007"""
    alles = 0
    for i in range(0,len(files)):
        if os.path.isfile(files[i]) and tarfile.is_tarfile(files[i]):
            tar = tarfile.open(files[i],'r:bz2')
            tar_names = tar.getnames();
            if (("s.bin" in tar_names) or (("s.txt" in tar_names))):
                s = numpy.asarray(lese_double_vektor_tar(tar,tar_names,"s"))
                if numpy.isscalar(alles):
                    alles = s
                else:
                    alles = numpy.hstack((alles,s))
            tar.close()
    smin = 0 # minimale Stabdicke
    smax = numpy.amax(alles) # maximale Stabdicke
    if minfaktor == 0:
        smin = numpy.amin(alles)
        if smin < 0:
            smin = 0
    else:
        smin = smax
        if not numpy.isscalar(alles):
            for s in alles:
                if (s >= minfaktor*smax) and (s < smin):
                    smin = s
    return [smin,smax]
def farben(n,typ=1):
    """f = farben(n,typ)
    returns a string of an RGB-color.
      typ == 1: The string has the format #%.2X%.2X%.2X with 0 <= %.2X <= 255.
                This is the color representation for gnuplot.
      typ == 2: The string has the format %f,%f,%f with 0 <= %f <= 1.
                This is the color representation for povray.
    
    Author: Daniel Mohr
    Date: 2012-02-15 (last change)
    License: GNU GENERAL PUBLIC LICENSE, Version 3, 29 June 2007"""
    from numpy import array,mod
    farben = array([[0.0, 0.0, 1.0],
                    [1.0, 0.0, 0.0],
                    [0.0, 1.0, 0.0],
                    [1.0, 0.0, 1.0],
                    [0.0, 1.0, 1.0],
                    [1.0, 1.0, 0.0],
                    [0.0, 0.0, 0.5],
                    [0.5, 0.0, 0.0],
                    [0.0, 0.5, 0.0],
                    [0.5, 0.0, 0.5],
                    [0.0, 0.5, 0.5],
                    [0.5, 0.5, 0.0]])
    n = mod(n,12)
    if typ == 1:
        f = '#%.2X%.2X%.2X' % (255*farben[n][0],255*farben[n][1],255*farben[n][2])
    else:
        f = '%f,%f,%f' % (farben[n][0],farben[n][1],farben[n][2])
    return f
def skaliere(s,smin,smax,typ=1):
    """Author: Daniel Mohr
    Date: 2012-03-06 (last change)
    License: GNU GENERAL PUBLIC LICENSE, Version 3, 29 June 2007"""
    # typ beschreibt den Typ der Skalierung:
    #  1 gnuplot
    #  2 povray
    #  3 gnuplot_cylinder
    if typ == 1:
        #s = skalierelinear(s,smin,smax,typ)
        s = skalieremultiplikativ(s,smin,smax,typ)
    else:
        s = skalieremultiplikativ(s,smin,smax,typ)
    return s
def skalieremultiplikativ(s,smin,smax,typ):
    """Author: Daniel Mohr
    Date: 2012-03-07 (last change)
    License: GNU GENERAL PUBLIC LICENSE, Version 3, 29 June 2007"""
    if typ == 1:
        if smin==smax:
            b = 3
        else:
            #mins = 1 # minimale Zeichendicke
            #b = mins/smin
            maxs = 30 # maximale Zeichendicke
            b = maxs/smax
    elif typ == 2:
        if smin==smax:
            b = 0.01
        else:
            maxs = 0.05 # maximale Zeichendicke
            b = maxs/smax
    elif typ == 3:
        if smin==smax:
            b = 0.02
        else:
            maxs = 0.1 # maximale Zeichendicke
            b = maxs/smax
    else:
        maxs = 0.05 # maximale Zeichendicke
        b = maxs/smax
    return round(b*s,3)
def skalierelinear(s,smin,smax,typ):
    """Author: Daniel Mohr
    Date: 2012-02-15 (last change)
    License: GNU GENERAL PUBLIC LICENSE, Version 3, 29 June 2007"""
    if typ == 1:
        maxs = 10 # maximale Zeichendicke
        mins = 1 # minimale Zeichendicke
    else:
        maxs = 0.1 # maximale Zeichendicke
        mins = 0.01 # minimale Zeichendicke
    b = (maxs-mins)/(smax-smin)
    a = maxs - b*smax
    return a+b*s
def create_png_pov(file,smin,smax,xaufloesung,yaufloesung,neu,args):
    """Author: Daniel Mohr
    Date: 2012-03-07 (last change)
    License: GNU GENERAL PUBLIC LICENSE, Version 3, 29 June 2007"""
    dd = os.path.splitext(file)[0]
    dd = os.path.splitext(dd)[0]
    if args.all_bars:
        dd += '_all_bars'
    ddpov = "%s.pov.pov" % (dd)
    if ((neu == False) or (os.path.isfile(ddpov) == False)) and os.path.isfile(file) and tarfile.is_tarfile(file):
        tar = tarfile.open(file,'r:bz2')
        tar_names = tar.getnames();
        if ((("s.bin" in tar_names) or (("s.txt" in tar_names))) and
            (("staebe.bin" in tar_names) or (("staebe.txt" in tar_names))) and
            (("knoten.bin" in tar_names) or (("knoten.txt" in tar_names))) and
            (("kraefte.bin" in tar_names) or (("kraefte.txt" in tar_names))) and
            (("lager.bin" in tar_names) or (("lager.txt" in tar_names))) and
            (("redundanz.bin" in tar_names) or (("redundanz.txt" in tar_names))) and
            (("dim.bin" in tar_names) or (("dim.txt" in tar_names)))):
            dim = lese_int_tar(tar,tar_names,"dim")
            if dim == 3:
                print('create png with povray for "%s"' % file)
                sys.stdout.flush()
                s = lese_double_vektor_tar(tar,tar_names,"s")
                if args.all_bars:
                    for i in range(0,len(s)):
                        s[i] = 1
                staebe = lese_int_matrix_tar(tar,tar_names,"staebe")
                sn = len(staebe)
                knoten = lese_double_matrix_tar(tar,tar_names,"knoten")
                # skaliere knoten auf sichtausschnitt
                knoten[:,0] = knoten[:,0] - numpy.amin(knoten[:,0])
                knoten[:,1] = knoten[:,1] - numpy.amin(knoten[:,1])
                knoten[:,2] = knoten[:,2] - numpy.amin(knoten[:,2])
                fak = 1.0 / numpy.amax(knoten)
                knoten = fak * knoten
                knoten[:,0] = knoten[:,0] - numpy.amax(knoten[:,0])/2
                knoten[:,1] = knoten[:,1] - numpy.amax(knoten[:,1])/2
                knoten[:,0] = knoten[:,0] + 0.5
                knoten[:,1] = knoten[:,1] + 0.5
                kraefte = lese_sparsematrix_tar(tar,tar_names,"kraefte")
                lager = lese_int_vektor_tar(tar,tar_names,"lager")
                redundanz = abs(lese_int_tar(tar,tar_names,"redundanz"))
                if redundanz == 0:
                    redundanz = 1
                fpov = open(ddpov,'w') # povray ausgabe
                fpov.write('// povray Width=%d Height=%d Display=0 Quality=9 Antialias=1 %s\n' % (xaufloesung,yaufloesung,ddpov))
                fpov.write('// Grundgeruest\n')
                fpov.write('#include "colors.inc"\n')
                fpov.write('global_settings {\n')
                fpov.write('  assumed_gamma 1\n')
                fpov.write('}\n')
                fpov.write('camera{\n')
                fpov.write(' orthographic\n')
                fpov.write(' right -x*image_width/image_height\n')
                fpov.write(' sky <0,0,1>// right handed !\n')
                fpov.write(' angle %d\n' % args.pov_angle)
                fpov.write(' location <2, -2, 2>\n')
                fpov.write(' look_at <0.5, 0.5, 0.5>\n')
                fpov.write(' rotate <0,0,0>\n')
                fpov.write('}\n')
                fpov.write('sky_sphere{\n')
                fpov.write(' pigment{\n')
                fpov.write('  gradient y\n')
                fpov.write('  color_map{\n')
                fpov.write('   [0.0 rgb <0.6,0.7,1.0>]\n')
                fpov.write('   [0.7 rgb <0.0,0.1,0.8>]\n')
                fpov.write('  }\n')
                fpov.write(' }\n')
                fpov.write('}\n')
                fpov.write('light_source{\n')
                fpov.write(' <0,-60,40>\n')
                fpov.write(' color rgb <1,1,1>\n')
                fpov.write('}\n')
                fpov.write('// Szeneninhalt\n')
                fpov.write('// Ebene\n')
                fpov.write('plane{\n')
                fpov.write(' z, 0\n')
                fpov.write(' pigment{\n')
                fpov.write('  color rgb <0.8,0.8,0.8>\n')
                fpov.write(' }\n')
                fpov.write('}\n')
                fpov.write('//// Koordinatensystem\n')
                fpov.write('//// x-Achse\n')
                fpov.write('//cylinder{<-0.5,0,0>,<1,0,0>,0.03\n')
                fpov.write('// texture{pigment{color rgb<1,0.3,0>}finish{ phong 1}}}\n')
                fpov.write('//cone{<1,0,0>,0.1,<1+0.3,0,0>,0\n')
                fpov.write('// texture{pigment{color rgb<1,0.3,0>}finish{ phong 1}}}\n')
                fpov.write('//// y-Achse\n')
                fpov.write('//cylinder{<0,-0.5,0>,<0,1,0>,0.03\n')
                fpov.write('// texture{pigment{color rgb<1,1,1>}finish{ phong 1}}}\n')
                fpov.write('//cone{<0,1,0>,0.1,<0,1+0.3,0>,0\n')
                fpov.write('// texture{pigment{color rgb<1,1,1>}finish{ phong 1}}}\n')
                fpov.write('//// z-Achse\n')
                fpov.write('//cylinder{<0,0,-0.5>,<0,0,1>,0.03\n')
                fpov.write('// texture{checker\n')
                fpov.write('//  texture{pigment{color rgb<1,0.3,0>}finish{ phong 1}}\n')
                fpov.write('//  texture{pigment{color rgb<1,1,1>}finish{ phong 1}}\n')
                fpov.write('//  translate<0.01,0,0.01>}}\n')
                fpov.write('//cone{<0,0,1>,0.1,<0,0,1+0.3>,0\n')
                fpov.write('// texture{checker\n')
                fpov.write('//  texture{pigment{color rgb<1,0.3,0>}finish{ phong 1}}\n')
                fpov.write('//  texture{pigment{color rgb<1,1,1>}finish{ phong 1}}\n')
                fpov.write('//  translate<0.01,0,0.01>}}\n')
                fpov.write('// Struktur\n')
                # zeichne
                j = 0
                for r in range(0,redundanz):
                    for i in range(0,sn):
                        if s[r*sn+i] >= smin:
                            fpov.write('cylinder { // Stab %d\n' % (j+1))
                            fpov.write('<%f,%f,%f>,<%f,%f,%f>,%f\n' % (knoten[staebe[i,0],0],knoten[staebe[i,0],1],knoten[staebe[i,0],2],knoten[staebe[i,1],0],knoten[staebe[i,1],1],knoten[staebe[i,1],2],skaliere(s[r*sn+i],smin,smax,2)))
                            fpov.write(' pigment{color rgb <%s>}\n' % farben(r,2))
                            fpov.write(' finish{phong 4}\n')
                            fpov.write('}\n')
                            j = j + 1
                os.fsync(fpov)
                fpov.close()
                prg = 'povray Width=%d Height=%d Display=0 Quality=9 Antialias=1 %s' % (xaufloesung,yaufloesung,ddpov)
                os.system('%s' % (prg))
        tar.close()
    else:
        print('"%s" is no tar-file or pictures/movies exists' % file)
        sys.stdout.flush()
    return
def create_movie_animation(file,smin,smax,xaufloesung,yaufloesung,neu,frames,framerate,args):
    """Author: Daniel Mohr
    Date: 2012-03-07 (last change)
    License: GNU GENERAL PUBLIC LICENSE, Version 3, 29 June 2007"""
    dd = os.path.splitext(file)[0]
    dd = os.path.splitext(dd)[0]
    if args.all_bars:
        dd += '_all_bars'
    ddpov = "%s.ogv" % (dd)
    if ((neu == False) or (os.path.isfile(ddpov) == False)) and os.path.isfile(file) and tarfile.is_tarfile(file):
        tar = tarfile.open(file,'r:bz2')
        tar_names = tar.getnames();
        if ((("s.bin" in tar_names) or (("s.txt" in tar_names))) and
            (("staebe.bin" in tar_names) or (("staebe.txt" in tar_names))) and
            (("knoten.bin" in tar_names) or (("knoten.txt" in tar_names))) and
            (("kraefte.bin" in tar_names) or (("kraefte.txt" in tar_names))) and
            (("lager.bin" in tar_names) or (("lager.txt" in tar_names))) and
            (("redundanz.bin" in tar_names) or (("redundanz.txt" in tar_names))) and
            (("dim.bin" in tar_names) or (("dim.txt" in tar_names))) and
            ("info.txt" in tar_names)):
            dim = lese_int_tar(tar,tar_names,"dim")
            if dim == 3:
                print('create ogv-movie with povray for "%s"' % file)
                sys.stdout.flush()
                s = lese_double_vektor_tar(tar,tar_names,"s")
                if args.all_bars:
                    for i in range(0,len(s)):
                        s[i] = 1
                staebe = lese_int_matrix_tar(tar,tar_names,"staebe")
                sn = len(staebe)
                knoten = lese_double_matrix_tar(tar,tar_names,"knoten")
                # skaliere knoten auf sichtausschnitt
                knoten[:,0] = knoten[:,0] - numpy.amin(knoten[:,0])
                knoten[:,1] = knoten[:,1] - numpy.amin(knoten[:,1])
                knoten[:,2] = knoten[:,2] - numpy.amin(knoten[:,2])
                fak = 1.0 / numpy.amax(knoten)
                knoten = fak * knoten
                knoten[:,0] = knoten[:,0] - numpy.amax(knoten[:,0])/2
                knoten[:,1] = knoten[:,1] - numpy.amax(knoten[:,1])/2
                kraefte = lese_sparsematrix_tar(tar,tar_names,"kraefte")
                lager = lese_int_vektor_tar(tar,tar_names,"lager")
                redundanz = abs(lese_int_tar(tar,tar_names,"redundanz"))
                if redundanz == 0:
                    redundanz = 1
                fd = tar.extractfile('info.txt')
                name = fd.readline()
                fd.close()
                # schreibe files
                #tmpfile = tempfile.NamedTemporaryFile(prefix='ttrr_tmp_',delete=False)
                tmpdir = tempfile.mkdtemp(prefix='ttrr_tmp_')
                ddpov = "%s/%s.pov" % (tmpdir,dd)
                fpov = open(ddpov,'w') # povray ausgabe
                fpov.write('// povray Width=%d Height=%d Display=0 Quality=9 Antialias=1 %s\n' % (xaufloesung,yaufloesung,ddpov))
                fpov.write('// Grundgeruest\n')
                fpov.write('#include "colors.inc"\n')
                fpov.write('global_settings {\n')
                fpov.write('  assumed_gamma 1\n')
                fpov.write('}\n')
                fpov.write('camera{\n')
                fpov.write(' orthographic\n')
                fpov.write(' right -x*image_width/image_height\n')
                fpov.write(' sky <0,0,1>// right handed !\n')
                fpov.write(' angle %d\n' % args.pov_angle_m)
                fpov.write(' location <2, -2, 2>\n')
                fpov.write(' look_at <0.5, 0.5, 0.5>\n')
                fpov.write(' rotate <0,0,0>\n')
                fpov.write(' translate<-0.5,-0.5,0>\n')
                #fpov.write(' translate<-%d,%d,0>\n' % (numpy.max(knoten[:,0]),numpy.max(knoten[:,1])))
                fpov.write('}\n')
                fpov.write('sky_sphere{\n')
                fpov.write(' pigment{\n')
                fpov.write('  gradient y\n')
                fpov.write('  color_map{\n')
                fpov.write('   [0.0 rgb <0.6,0.7,1.0>]\n')
                fpov.write('   [0.7 rgb <0.0,0.1,0.8>]\n')
                fpov.write('  }\n')
                fpov.write(' }\n')
                fpov.write('}\n')
                fpov.write('light_source{\n')
                fpov.write(' <0,-60,40>\n')
                fpov.write(' color rgb <1,1,1>\n')
                fpov.write('}\n')
                fpov.write('// Szeneninhalt\n')
                fpov.write('// Ebene\n')
                fpov.write('plane{\n')
                fpov.write(' z, 0\n')
                fpov.write(' pigment{\n')
                fpov.write('  color rgb <0.8,0.8,0.8>\n')
                fpov.write(' }\n')
                fpov.write('}\n')
                fpov.write('//// Koordinatensystem\n')
                fpov.write('//// x-Achse\n')
                fpov.write('//cylinder{<-0.5,0,0>,<1,0,0>,0.03\n')
                fpov.write('// texture{pigment{color rgb<1,0.3,0>}finish{ phong 1}}}\n')
                fpov.write('//cone{<1,0,0>,0.1,<1+0.3,0,0>,0\n')
                fpov.write('// texture{pigment{color rgb<1,0.3,0>}finish{ phong 1}}}\n')
                fpov.write('//// y-Achse\n')
                fpov.write('//cylinder{<0,-0.5,0>,<0,1,0>,0.03\n')
                fpov.write('// texture{pigment{color rgb<1,1,1>}finish{ phong 1}}}\n')
                fpov.write('//cone{<0,1,0>,0.1,<0,1+0.3,0>,0\n')
                fpov.write('// texture{pigment{color rgb<1,1,1>}finish{ phong 1}}}\n')
                fpov.write('//// z-Achse\n')
                fpov.write('//cylinder{<0,0,-0.5>,<0,0,1>,0.03\n')
                fpov.write('// texture{checker\n')
                fpov.write('//  texture{pigment{color rgb<1,0.3,0>}finish{ phong 1}}\n')
                fpov.write('//  texture{pigment{color rgb<1,1,1>}finish{ phong 1}}\n')
                fpov.write('//  translate<0.01,0,0.01>}}\n')
                fpov.write('//cone{<0,0,1>,0.1,<0,0,1+0.3>,0\n')
                fpov.write('// texture{checker\n')
                fpov.write('//  texture{pigment{color rgb<1,0.3,0>}finish{ phong 1}}\n')
                fpov.write('//  texture{pigment{color rgb<1,1,1>}finish{ phong 1}}\n')
                fpov.write('//  translate<0.01,0,0.01>}}\n')
                fpov.write('// Struktur\n')
                fpov.write('union{\n')
                # zeichne
                j = 0
                for r in range(0,redundanz):
                    for i in range(0,sn):
                        if s[r*sn+i] >= smin:
                            fpov.write(' cylinder { // Stab %d\n' % (j+1))
                            fpov.write('  <%f,%f,%f>,<%f,%f,%f>,%f\n' % (knoten[staebe[i,0],0],knoten[staebe[i,0],1],knoten[staebe[i,0],2],knoten[staebe[i,1],0],knoten[staebe[i,1],1],knoten[staebe[i,1],2],skaliere(s[r*sn+i],smin,smax,2)))
                            fpov.write('  pigment{color rgb <%s>}\n' % farben(r,2))
                            fpov.write('  finish{phong 4}\n')
                            fpov.write(' }\n')
                            j = j + 1
                #fpov.write(' translate<-0.5,-0.5,0>\n')
                #fpov.write(' translate<-%d,-%d,0>\n' % (numpy.max(knoten[:,0])/2,numpy.max(knoten[:,1])/2))
                fpov.write(' rotate <0,0,0-360*clock>\n')
                fpov.write('}\n')
                os.fsync(fpov)
                fpov.close()
                prg = 'povray Width=%d Height=%d Display=0 Quality=9 Antialias=1 Initial_Frame=0 Final_Frame=%d Initial_Clock=0 Final_Clock=1 Cyclic_Animation=1 Output_File_Name=%s/a' % (xaufloesung,yaufloesung,frames,tmpdir)
                p1 = subprocess.Popen('%s Subset_Start_Frame=%d Subset_End_Frame=%d %s' % (prg,0*numpy.floor(frames/3)+0,1*numpy.floor(frames/3),ddpov),bufsize=-1,shell=True)
                p2 = subprocess.Popen('%s Subset_Start_Frame=%d Subset_End_Frame=%d %s' % (prg,1*numpy.floor(frames/3)+1,2*numpy.floor(frames/3),ddpov),bufsize=-1,shell=True)
                p3 = subprocess.Popen('%s Subset_Start_Frame=%d Subset_End_Frame=%d %s' % (prg,2*numpy.floor(frames/3)+1,frames,ddpov),bufsize=-1,shell=True)
                p1.wait()
                p2.wait()
                p3.wait()
                if args.yuv or (args.ogv and (args.ogv_convert == 'ffmpeg2theora')):
                    prg = 'png2yuv -I p -f %d -n %d -j %s/a%%0%dd.png -b 1 > %s/tmp.yuv' % (framerate,frames,tmpdir,int(numpy.log10(frames+1)+1),tmpdir)
                    p1 = subprocess.Popen(prg,bufsize=-1,shell=True)
                    p1.wait()
                    if args.yuv:
                        os.system('cp -p %s/tmp.yuv %s.yuv' % (tmpdir,dd))
                if args.ogv:
                    if args.ogv_convert == 'ffmpeg2theora':
                        prg = 'ffmpeg2theora --noaudio --optimize --videoquality 10 --videobitrate 16778 -o %s.ogv --title "%s" --date "%s" --copyright "%s" --artist "ttrr" %s/tmp.yuv' % (dd,name,datetime.now().isoformat(),os.getenv("USER"),tmpdir)
                        p1 = subprocess.Popen(prg,bufsize=-1,shell=True)
                    elif args.ogv_convert == 'png2theora':
                        prg = 'png2theora --output %s.ogv --video-quality 10 --video-rate-target 16778 --keyframe-freq %d %s/a%%0%dd.png' % (dd,framerate,tmpdir,int(numpy.log10(frames+1)+1))
                        p1 = subprocess.Popen(prg,bufsize=-1,shell=True)
                    elif args.ogv_convert == 'ffmpeg':
                        prg = 'ffmpeg -i %s/a%%0%dd.png -metadata title="%s" -timestamp now -metadata copyright="%s" -metadata artist="ttrr" -r %d -b 1717248 %s.ogv' % (tmpdir,int(numpy.log10(frames+1)+1),name,os.getenv("USER"),framerate,dd)
                        p1 = subprocess.Popen(prg,bufsize=-1,shell=True)
                if args.apng:
                    prg = 'apngasm %s.apng.png %s/a*.png 1 %d' % (dd,tmpdir,framerate)
                    p2 = subprocess.Popen(prg,bufsize=-1,shell=True)
                if args.mng:
                    prg = 'convert -comment "%s" -delay %d %s/a*.png %s.mng' % (name,int(100.0/framerate),tmpdir,dd)
                    p3 = subprocess.Popen(prg,bufsize=-1,shell=True)
                if args.gif:
                    prg = 'convert -comment "%s" -delay %d -loop 0 %s/a*.png %s.gif' % (name,int(100.0/framerate),tmpdir,dd)
                    p4 = subprocess.Popen(prg,bufsize=-1,shell=True)
                if args.ogv:
                    p1.wait()
                if args.apng:
                    p2.wait()
                if args.mng:
                    p3.wait()
                if args.gif:
                    p4.wait()
                for d in os.listdir(tmpdir):
                    os.unlink('%s/%s' % (tmpdir,d))
                os.rmdir(tmpdir)
        tar.close()
    else:
        print('"%s" is no tar-file or pictures/movies exists' % file)
        sys.stdout.flush()
    return
def killprocess(p):
    """Author: Daniel Mohr
    Date: 2012-02-15 (last change)
    License: GNU GENERAL PUBLIC LICENSE, Version 3, 29 June 2007"""
    try:
        p.terminate()
        p.poll()
    except:
        try:
            p.kill()
            p.poll()
        except:
            print('can not kill process (pid = %d)' % p.pid)
    return
def visualisation(args):
    """Author: Daniel Mohr
    Date: 2012-03-07 (last change)
    License: GNU GENERAL PUBLIC LICENSE, Version 3, 29 June 2007"""
    gnuplot_exists = False
    gnuplot_pdf = False
    gnuplot_png = False
    if numpy.isscalar(args.x):
        xaufloesung = args.x
    else:
        xaufloesung = args.x[0]
    if numpy.isscalar(args.y):
        yaufloesung = args.y
    else:
        yaufloesung = args.y[0]
    if numpy.isscalar(args.xm):
        xaufloesungm = args.xm
    else:
        xaufloesungm = args.xm[0]
    if numpy.isscalar(args.ym):
        yaufloesungm = args.ym
    else:
        yaufloesungm = args.ym[0]
    if numpy.isscalar(args.minfaktor):
        minfaktor = args.minfaktor # stabdicken kleiner als minfaktor*max(s) nicht beruecksichtigen; der Wert 0 sorgt dafuer, dass alles beruecksichtigt wird
    else:
        minfaktor = args.minfaktor[0] # stabdicken kleiner als minfaktor*max(s) nicht beruecksichtigen; der Wert 0 sorgt dafuer, dass alles beruecksichtigt wird
    if numpy.isscalar(args.frames):
        frames = args.frames
    else:
        frames = args.frames[0]
    if numpy.isscalar(args.framerate):
        framerate = args.framerate
    else:
        framerate = args.framerate[0]
    if not numpy.isscalar(args.pov_angle):
        args.pov_angle = args.pov_angle[0]
    if not numpy.isscalar(args.pov_angle_m):
        args.pov_angle_m = args.pov_angle_m[0]
    if args.pngs_gnu or args.pdfs_gnu:
        # check gnuplot
        p1 = subprocess.Popen('gnuplot -version',bufsize=-1,shell=True,stdin=subprocess.PIPE,stdout=subprocess.PIPE,stderr=subprocess.PIPE)
        it = 0
        while p1.poll()==None:
            if it > 10:
                killprocess(p1)
            time.sleep(0.1)
            it += 1
        if (0<=p1.returncode) and (p1.returncode<=10):
            gnuplot_exists = True
        else:
            print('p1.returncode=%d' % p1.returncode)
        if gnuplot_exists:
            p2 = subprocess.Popen('gnuplot -e "set terminal pdf"',bufsize=-1,shell=True,stdin=subprocess.PIPE,stdout=subprocess.PIPE,stderr=subprocess.PIPE)
            it = 0
            while p2.poll()==None:
                if it > 10:
                    killprocess(p2)
                time.sleep(0.1)
                it += 1
            if p2.returncode == 0:
                gnuplot_pdf = True
            p2 = subprocess.Popen('gnuplot -e "set terminal png"',bufsize=-1,shell=True,stdin=subprocess.PIPE,stdout=subprocess.PIPE,stderr=subprocess.PIPE)
            it = 0
            while p2.poll()==None:
                if it > 10:
                    killprocess(p2)
                time.sleep(0.1)
                it += 1
            if p2.returncode == 0:
                gnuplot_png = True
    if (args.pngs_gnu == False) and (gnuplot_png == True):
        gnuplot_png = False
    if (args.pdfs_gnu == False) and (gnuplot_pdf == True):
        gnuplot_pdf = False
    if (gnuplot_pdf or gnuplot_png) or args.pngs_pov or args.ogv or args.apng or args.mng or args.gif or args.yuv:
        if args.all_bars:
            smin = 1
            smax = 1
        else:
            [smin,smax] = scan_files(args.file,minfaktor)
        for i in range(0,len(args.file)):
            if (gnuplot_pdf or gnuplot_png) and args.gnuplot_cylinder==False:
                create_png_pdf_gnuplot_simple(args.file[i],smin,smax,xaufloesung,yaufloesung,args.new,gnuplot_pdf,gnuplot_png,args)
            if (gnuplot_pdf or gnuplot_png) and args.gnuplot_cylinder:
                create_png_pdf_gnuplot(args.file[i],smin,smax,xaufloesung,yaufloesung,args.new,gnuplot_pdf,gnuplot_png,args)
            if args.pngs_pov:
                create_png_pov(args.file[i],smin,smax,xaufloesung,yaufloesung,args.new,args)
            if args.ogv or args.apng or args.mng or args.gif or args.yuv:
                create_movie_animation(args.file[i],smin,smax,xaufloesungm,yaufloesungm,args.new,frames,framerate,args)
